@@ -21,9 +21,8 @@ ENCODING = 'utf-8'
 
 if PY3:
 
-    def bstr(x, encoding=ENCODING):
-        return bytes(x, encoding=encoding)
-
+    def bstr(value, encoding=ENCODING):
+        return bytes(value, encoding=encoding)
 
 elif PY26_PLUS:
     bstr = bytes
@@ -43,16 +42,13 @@ if PY3:
     uchr = chr
 else:
     bchr = chr
-    maxsize = 2 ** 31
-    # pylint: disable=unicode-builtin
+    maxsize = 2**31
     ustr = unicode  # noqa
-    # pylint: disable=unichr-builtin
     uchr = unichr  # noqa
-    # pylint: disable=long-builtin
     int_types = (int, long)  # noqa
 
 # Qt's max 32-bit signed integer range (-2147483648 to 2147483647)
-maxint = (2 ** 31) - 1
+maxint = (2**31) - 1
 
 
 def setenv(key, value):
