@@ -60,7 +60,6 @@ class UStr(ustr):
     """
 
     def __new__(cls, string, encoding):
-
         if isinstance(string, UStr):
             if encoding != string.encoding:
                 raise ValueError(
@@ -142,9 +141,13 @@ def read(filename, size=-1, encoding=None, errors='strict'):
         return xread(fh, size=size, encoding=encoding, errors=errors)
 
 
-def write(path, contents, encoding=None):
+def write(path, contents, encoding=None, append=False):
     """Writes a unicode string to a file"""
-    with xopen(path, 'wb') as fh:
+    if append:
+        mode = 'ab'
+    else:
+        mode = 'wb'
+    with xopen(path, mode) as fh:
         return xwrite(fh, contents, encoding=encoding)
 
 
@@ -357,24 +360,24 @@ def _decorator_noop(x):
     return x
 
 
-def wrap(action, fn, decorator=None):
+def wrap(action, func, decorator=None):
     """Wrap arguments with `action`, optionally decorate the result"""
     if decorator is None:
         decorator = _decorator_noop
 
-    @functools.wraps(fn)
+    @functools.wraps(func)
     def wrapped(*args, **kwargs):
-        return decorator(fn(action(*args, **kwargs)))
+        return decorator(func(action(*args, **kwargs)))
 
     return wrapped
 
 
-def decorate(decorator, fn):
-    """Decorate the result of `fn` with `action`"""
+def decorate(decorator, func):
+    """Decorate the result of `func` with `action`"""
 
-    @functools.wraps(fn)
+    @functools.wraps(func)
     def decorated(*args, **kwargs):
-        return decorator(fn(*args, **kwargs))
+        return decorator(func(*args, **kwargs))
 
     return decorated
 
@@ -396,9 +399,29 @@ def guess_mimetype(filename):
 
 
 def xopen(path, mode='r', encoding=None):
+    """Open a file with the specified mode and encoding
+
+    The path is decoded into unicode on Windows and encoded into bytes on Unix.
+    """
+    # pylint: disable=unspecified-encoding
     return open(mkpath(path, encoding=encoding), mode)
 
 
+def open_append(path, encoding=None):
+    """Open a file for appending in utf-8 text mode"""
+    return open(mkpath(path, encoding=encoding), 'a', encoding='utf-8')
+
+
+def open_read(path, encoding=None):
+    """Open a file for reading in utf-8 text mode"""
+    return open(mkpath(path, encoding=encoding), 'rt', encoding='utf-8')
+
+
+def open_write(path, encoding=None):
+    """Open a file for writing in utf-8 text mode"""
+    return open(mkpath(path, encoding=encoding), 'wt', encoding='utf-8')
+
+
 def print_stdout(msg, linesep='\n'):
     msg = msg + linesep
     if PY2:
@@ -455,11 +478,11 @@ def _find_executable(executable, path=None):
         executable = executable + '.exe'
 
     if not os.path.isfile(executable):
-        for p in paths:
-            f = os.path.join(p, executable)
-            if os.path.isfile(f):
+        for dirname in paths:
+            filename = os.path.join(dirname, executable)
+            if os.path.isfile(filename):
                 # the file exists, we have a shot at spawn working
-                return f
+                return filename
         return None
 
     return executable
@@ -499,4 +522,7 @@ relpath = wrap(mkpath, os.path.relpath, decorator=decode)
 remove = wrap(mkpath, os.remove)
 stat = wrap(mkpath, os.stat)
 unlink = wrap(mkpath, os.unlink)
-walk = wrap(mkpath, os.walk)
+if PY2:
+    walk = wrap(mkpath, os.walk)
+else:
+    walk = os.walk
