@@ -37,18 +37,18 @@ def _cache_key_from_paths(paths):
     return None
 
 
-def _config_to_python(v):
+def _config_to_python(value):
     """Convert a Git config string into a Python value"""
-    if v in ('true', 'yes'):
-        v = True
-    elif v in ('false', 'no'):
-        v = False
+    if value in ('true', 'yes'):
+        value = True
+    elif value in ('false', 'no'):
+        value = False
     else:
         try:
-            v = int(v)  # pylint: disable=redefined-variable-type
+            value = int(value)
         except ValueError:
             pass
-    return v
+    return value
 
 
 def unhex(value):
@@ -158,7 +158,7 @@ class GitConfig(QtCore.QObject):
         worktree_scope = 'worktree'
         cache_paths = set()
 
-        for (current_scope, current_key, current_value, continuation) in reader(
+        for current_scope, current_key, current_value, continuation in reader(
             self.context, cache_paths, self._renamed_keys
         ):
             # Store the values for fast cached lookup.
@@ -193,14 +193,14 @@ class GitConfig(QtCore.QObject):
         # Send a notification that the configuration has been updated.
         self.updated.emit()
 
-    def _get(self, src, key, default, fn=None, cached=True):
+    def _get(self, src, key, default, func=None, cached=True):
         if not cached or not src:
             self.update()
         try:
             value = self._get_value(src, key)
         except KeyError:
-            if fn:
-                value = fn()
+            if func:
+                value = func()
             else:
                 value = default
         return value
@@ -220,9 +220,9 @@ class GitConfig(QtCore.QObject):
         # Allow the final KeyError to bubble up
         return src[key.lower()]
 
-    def get(self, key, default=None, fn=None, cached=True):
+    def get(self, key, default=None, func=None, cached=True):
         """Return the string value for a config key."""
-        return self._get(self._all, key, default, fn=fn, cached=cached)
+        return self._get(self._all, key, default, func=func, cached=cached)
 
     def get_all(self, key):
         """Return all values for a key sorted in priority order
@@ -311,7 +311,7 @@ class GitConfig(QtCore.QObject):
 
     def is_per_file_attrs_enabled(self):
         return self.get(
-            'cola.fileattributes', fn=lambda: os.path.exists('.gitattributes')
+            'cola.fileattributes', func=lambda: os.path.exists('.gitattributes')
         )
 
     def is_binary(self, path):
@@ -382,43 +382,52 @@ class GitConfig(QtCore.QObject):
         return [(name, self.get('guitool.%s.shortcut' % name)) for name in names]
 
     def terminal(self):
+        """Return a suitable terminal command for running a shell"""
         term = self.get('cola.terminal', default=None)
-        if not term:
-            # find a suitable default terminal
-            term = 'xterm -e'  # for mac osx
-            if utils.is_win32():
-                # Try to find Git's sh.exe directory in
-                # one of the typical locations
-                pf = os.environ.get('ProgramFiles', r'C:\Program Files')
-                pf32 = os.environ.get('ProgramFiles(x86)', r'C:\Program Files (x86)')
-                pf64 = os.environ.get('ProgramW6432', r'C:\Program Files')
+        if term:
+            return term
 
-                for p in [pf64, pf32, pf, 'C:\\']:
-                    candidate = os.path.join(p, r'Git\bin\sh.exe')
-                    if os.path.isfile(candidate):
-                        return candidate
-                return None
-            else:
-                candidates = ('xfce4-terminal', 'konsole', 'gnome-terminal')
-                for basename in candidates:
-                    if core.exists('/usr/bin/%s' % basename):
-                        if basename == 'gnome-terminal':
-                            term = '%s --' % basename
-                        else:
-                            term = '%s -e' % basename
-                        break
-        return term
+        # find a suitable default terminal
+        if utils.is_win32():
+            # Try to find Git's sh.exe directory in
+            # one of the typical locations
+            pf = os.environ.get('ProgramFiles', r'C:\Program Files')
+            pf32 = os.environ.get('ProgramFiles(x86)', r'C:\Program Files (x86)')
+            pf64 = os.environ.get('ProgramW6432', r'C:\Program Files')
+
+            for p in [pf64, pf32, pf, 'C:\\']:
+                candidate = os.path.join(p, r'Git\bin\sh.exe')
+                if os.path.isfile(candidate):
+                    return candidate
+            return None
+
+        # If no terminal has been configured then we'll look for the following programs
+        # and use the first one we find.
+        terminals = (
+            # (<executable>, <command> for running arbitrary commands)
+            ('kitty', 'kitty'),
+            ('alacritty', 'alacritty -e'),
+            ('uxterm', 'uxterm -e'),
+            ('konsole', 'konsole -e'),
+            ('gnome-terminal', 'gnome-terminal --'),
+            ('mate-terminal', 'mate-terminal --'),
+            ('xterm', 'xterm -e'),
+        )
+        for executable, command in terminals:
+            if core.find_executable(executable):
+                return command
+        return None
 
     def color(self, key, default):
         value = self.get('cola.color.%s' % key, default=default)
         struct_layout = core.encode('BBB')
         try:
             # pylint: disable=no-member
-            r, g, b = struct.unpack(struct_layout, unhex(value))
+            red, green, blue = struct.unpack(struct_layout, unhex(value))
         except (struct.error, TypeError):
             # pylint: disable=no-member
-            r, g, b = struct.unpack(struct_layout, unhex(default))
-        return (r, g, b)
+            red, green, blue = struct.unpack(struct_layout, unhex(default))
+        return (red, green, blue)
 
     def hooks(self):
         """Return the path to the git hooks directory"""
