@@ -15,29 +15,28 @@ from .i18n import N_
 from .interaction import Interaction
 
 
-class InvalidRepositoryError(Exception):
-    pass
-
-
 def add(context, items, u=False):
     """Run "git add" while preventing argument overflow"""
-    fn = context.git.add
-    return utils.slice_fn(
-        items, lambda paths: fn('--', force=True, verbose=True, u=u, *paths)
+    git_add = context.git.add
+    return utils.slice_func(
+        items, lambda paths: git_add('--', force=True, verbose=True, u=u, *paths)
     )
 
 
 def apply_diff(context, filename):
+    """Use "git apply" to apply the patch in `filename` to the staging area"""
     git = context.git
     return git.apply(filename, index=True, cached=True)
 
 
 def apply_diff_to_worktree(context, filename):
+    """Use "git apply" to apply the patch in `filename` to the worktree"""
     git = context.git
     return git.apply(filename)
 
 
 def get_branch(context, branch):
+    """Get the current branch"""
     if branch is None:
         branch = current_branch(context)
     return branch
@@ -151,7 +150,7 @@ def all_files(context, *args):
     return sorted([f for f in ls_files.split('\0') if f])
 
 
-class _current_branch(object):
+class CurrentBranchCache(object):
     """Cache for current_branch()"""
 
     key = None
@@ -159,7 +158,8 @@ class _current_branch(object):
 
 
 def reset():
-    _current_branch.key = None
+    """Reset cached value in this module (eg. the cached current branch)"""
+    CurrentBranchCache.key = None
 
 
 def current_branch(context):
@@ -168,8 +168,8 @@ def current_branch(context):
     head = git.git_path('HEAD')
     try:
         key = core.stat(head).st_mtime
-        if _current_branch.key == key:
-            return _current_branch.value
+        if CurrentBranchCache.key == key:
+            return CurrentBranchCache.value
     except OSError:
         # OSError means we can't use the stat cache
         key = 0
@@ -182,8 +182,8 @@ def current_branch(context):
     for refs_prefix in ('refs/heads/', 'refs/remotes/', 'refs/tags/'):
         if data.startswith(refs_prefix):
             value = data[len(refs_prefix) :]
-            _current_branch.key = key
-            _current_branch.value = value
+            CurrentBranchCache.key = key
+            CurrentBranchCache.value = value
             return value
     # Detached head
     return data
@@ -202,7 +202,7 @@ def _read_git_head(context, head, default='main'):
         # Detached head
         return data
     # Legacy .git/HEAD symlinks
-    elif islink:
+    if islink:
         refs_heads = core.realpath(git.git_path('refs', 'heads'))
         path = core.abspath(head).replace('\\', '/')
         if path.startswith(refs_heads + '/'):
@@ -242,8 +242,9 @@ def for_each_ref_basename(context, refs):
     return [x[offset:] for x in non_heads]
 
 
-def _triple(x, y):
-    return (x, len(x) + 1, y)
+def _prefix_and_size(prefix, values):
+    """Return a tuple of (prefix, len(prefix) + 1, y) for <prefix>/ stripping"""
+    return (prefix, len(prefix) + 1, values)
 
 
 def all_refs(context, split=False, sort_key='version:refname'):
@@ -252,11 +253,10 @@ def all_refs(context, split=False, sort_key='version:refname'):
     local_branches = []
     remote_branches = []
     tags = []
-    triple = _triple
     query = (
-        triple('refs/tags', tags),
-        triple('refs/heads', local_branches),
-        triple('refs/remotes', remote_branches),
+        _prefix_and_size('refs/tags', tags),
+        _prefix_and_size('refs/heads', local_branches),
+        _prefix_and_size('refs/remotes', remote_branches),
     )
     sort = _version_sort(context, key=sort_key)
     _, out, _ = git.for_each_ref(format='%(refname)', sort=sort, _readonly=True)
@@ -361,6 +361,7 @@ def common_diff_opts(context):
         'no_ext_diff': True,
         'unified': config.get('gui.diffcontext', default=3),
         '_raw': True,
+        '_readonly': True,
     }
     opts.update(_diff_overrides)
     return opts
@@ -376,27 +377,38 @@ def oid_diff(context, oid, filename=None):
     # Naively "$oid^!" is what we'd like to use but that doesn't
     # give the correct result for merges--the diff is reversed.
     # Be explicit and compare oid against its first parent.
+    return oid_diff_range(context, oid + '~', oid, filename=filename)
+
+
+def oid_diff_range(context, start, end, filename=None):
+    """Reeturn the diff for a commit range"""
+    args = [start, end]
     git = context.git
-    args = [oid + '~', oid]
     opts = common_diff_opts(context)
     _add_filename(args, filename)
     status, out, _ = git.diff(*args, **opts)
     if status != 0:
         # We probably don't have "$oid~" because this is the root commit.
         # "git show" is clever enough to handle the root commit.
-        args = [oid + '^!']
+        args = [end + '^!']
         _add_filename(args, filename)
-        _, out, _ = git.show(pretty='format:', _readonly=True, *args, **opts)
+        _, out, _ = git.show(pretty='format:', *args, **opts)
         out = out.lstrip()
     return out
 
 
 def diff_info(context, oid, filename=None):
+    """Return the diff for the specified oid"""
+    return diff_range(context, oid + '~', oid, filename=filename)
+
+
+def diff_range(context, start, end, filename=None):
+    """Return the diff for the specified commit range"""
     git = context.git
-    decoded = log(git, '-1', oid, '--', pretty='format:%b').strip()
+    decoded = log(git, '-1', end, '--', pretty='format:%b').strip()
     if decoded:
         decoded += '\n\n'
-    return decoded + oid_diff(context, oid, filename=filename)
+    return decoded + oid_diff_range(context, start, end, filename=filename)
 
 
 # pylint: disable=too-many-arguments
@@ -572,8 +584,8 @@ def export_patchset(context, start, end, output='patches', **kwargs):
 def reset_paths(context, head, items):
     """Run "git reset" while preventing argument overflow"""
     items = list(set(items))
-    fn = context.git.reset
-    status, out, err = utils.slice_fn(items, lambda paths: fn(head, '--', *paths))
+    func = context.git.reset
+    status, out, err = utils.slice_func(items, lambda paths: func(head, '--', *paths))
     return (status, out, err)
 
 
@@ -722,7 +734,6 @@ def list_submodule(context):
     if status == 0 and data:
         data = data.splitlines()
         # see git submodule status
-        # TODO better separation
         for line in data:
             state = line[0].strip()
             sha1 = line[1 : OID_LENGTH + 1]
@@ -748,32 +759,6 @@ def merge_base_parent(context, branch):
     return 'HEAD'
 
 
-# TODO Unused?
-def parse_ls_tree(context, rev):
-    """Return a list of (mode, type, oid, path) tuples."""
-    output = []
-    git = context.git
-    lines = git.ls_tree(rev, r=True, _readonly=True)[STDOUT].splitlines()
-    regex = re.compile(r'^(\d+)\W(\w+)\W(\w+)[ \t]+(.*)$')
-    for line in lines:
-        match = regex.match(line)
-        if match:
-            mode = match.group(1)
-            objtype = match.group(2)
-            oid = match.group(3)
-            filename = match.group(4)
-            output.append(
-                (
-                    mode,
-                    objtype,
-                    oid,
-                    filename,
-                )
-            )
-    return output
-
-
-# TODO unused?
 def ls_tree(context, path, ref='HEAD'):
     """Return a parsed git ls-tree result for a single directory"""
     git = context.git
@@ -870,24 +855,60 @@ def prepare_commit_message_hook(context):
     return config.get('cola.preparecommitmessagehook', default=default_hook)
 
 
-def abort_merge(context):
-    """Abort a merge by reading the tree at HEAD."""
+def cherry_pick(context, revs):
+    """Cherry-picks each revision into the current branch.
+
+    Returns (0, out, err) where stdout and stderr across all "git cherry-pick"
+    invocations are combined into single values when all cherry-picks succeed.
+
+    Returns a combined (status, out, err) of the first failing "git cherry-pick"
+    in the event of a non-zero exit status.
+    """
+    if not revs:
+        return []
+    outs = []
+    errs = []
+    status = 0
+    for rev in revs:
+        status, out, err = context.git.cherry_pick(rev)
+        if status != 0:
+            details = N_(
+                'Hint: The "Actions > Abort Cherry-Pick" menu action can be used to '
+                'cancel the current cherry-pick.'
+            )
+            output = '# git cherry-pick %s\n# %s\n\n%s' % (rev, details, out)
+            return (status, output, err)
+        outs.append(out)
+        errs.append(err)
+    return (0, '\n'.join(outs), '\n'.join(errs))
+
+
+def abort_apply_patch(context):
+    """Abort a "git am" session."""
     # Reset the worktree
     git = context.git
-    status, out, err = git.read_tree('HEAD', reset=True, u=True, v=True, _readonly=True)
-    # remove MERGE_HEAD
-    merge_head = git.git_path('MERGE_HEAD')
-    if core.exists(merge_head):
-        core.unlink(merge_head)
-    # remove MERGE_MESSAGE, etc.
-    merge_msg_path = merge_message_path(context)
-    while merge_msg_path:
-        core.unlink(merge_msg_path)
-        merge_msg_path = merge_message_path(context)
+    status, out, err = git.am(abort=True)
+    return status, out, err
+
+
+def abort_cherry_pick(context):
+    """Abort a cherry-pick."""
+    # Reset the worktree
+    git = context.git
+    status, out, err = git.cherry_pick(abort=True)
+    return status, out, err
+
+
+def abort_merge(context):
+    """Abort a merge"""
+    # Reset the worktree
+    git = context.git
+    status, out, err = git.merge(abort=True)
     return status, out, err
 
 
 def strip_remote(remotes, remote_branch):
+    """Get branch names with the "<remote>/" prefix removed"""
     for remote in remotes:
         prefix = remote + '/'
         if remote_branch.startswith(prefix):
@@ -938,10 +959,12 @@ def write_blob(context, oid, filename):
 
 
 def cat_file_blob(context, filename, oid):
+    """Write a blob from git to the specified filename"""
     return cat_file(context, filename, 'blob', oid)
 
 
 def cat_file_to_path(context, filename, oid):
+    """Extract a file from an commit ref and a write it to the specified filename"""
     return cat_file(context, filename, oid, path=filename, filters=True)
 
 
@@ -954,9 +977,9 @@ def cat_file(context, filename, *args, **kwargs):
     basename = os.path.basename(filename)
     suffix = '-' + basename  # ensures the correct filename extension
     path = utils.tmp_filename('blob', suffix=suffix)
-    with open(path, 'wb') as fp:
+    with open(path, 'wb') as tmp_file:
         status, out, err = git.cat_file(
-            _raw=True, _readonly=True, _stdout=fp, *args, **kwargs
+            _raw=True, _readonly=True, _stdout=tmp_file, *args, **kwargs
         )
         Interaction.command(N_('Error'), 'git cat-file', status, out, err)
         if status == 0:
@@ -1003,6 +1026,7 @@ def annex_path(context, head, filename):
 
 
 def is_binary(context, filename):
+    """A heustic to determine whether `filename` contains (non-text) binary content"""
     cfg_is_binary = context.cfg.is_binary(filename)
     if cfg_is_binary is not None:
         return cfg_is_binary
