@@ -14,6 +14,7 @@ AUTOCOMPLETE_PATHS = 'cola.autocompletepaths'
 AUTOTEMPLATE = 'cola.autoloadcommittemplate'
 BACKGROUND_EDITOR = 'cola.backgroundeditor'
 BLAME_VIEWER = 'cola.blameviewer'
+BLOCK_CURSOR = 'cola.blockcursor'
 BOLD_HEADERS = 'cola.boldheaders'
 CHECK_CONFLICTS = 'cola.checkconflicts'
 CHECK_PUBLISHED_COMMITS = 'cola.checkpublishedcommits'
@@ -29,12 +30,15 @@ HIDPI = 'cola.hidpi'
 HISTORY_BROWSER = 'gui.historybrowser'
 ICON_THEME = 'cola.icontheme'
 LINEBREAK = 'cola.linebreak'
+LOGDATE = 'cola.logdate'
 MAXRECENT = 'cola.maxrecent'
 MERGE_DIFFSTAT = 'merge.diffstat'
 MERGE_KEEPBACKUP = 'merge.keepbackup'
 MERGE_SUMMARY = 'merge.summary'
 MERGE_VERBOSITY = 'merge.verbosity'
 MERGETOOL = 'merge.tool'
+MOUSE_ZOOM = 'cola.mousezoom'
+PATCHES_DIRECTORY = 'cola.patchesdirectory'
 RESIZE_BROWSER_COLUMNS = 'cola.resizebrowsercolumns'
 SAFE_MODE = 'cola.safemode'
 SAVEWINDOWSETTINGS = 'cola.savewindowsettings'
@@ -50,12 +54,41 @@ USER_EMAIL = 'user.email'
 USER_NAME = 'user.name'
 
 
+class DateFormat(object):
+    DEFAULT = 'default'
+    RELATIVE = 'relative'
+    LOCAL = 'local'
+    ISO = 'iso8601'
+    ISO_STRICT = 'iso8601-strict'
+    RFC = 'rfc2822'
+    SHORT = 'short'
+    RAW = 'raw'
+    HUMAN = 'human'
+    UNIX = 'unix'
+
+
+def date_formats():
+    return [
+        DateFormat.DEFAULT,
+        DateFormat.RELATIVE,
+        DateFormat.LOCAL,
+        DateFormat.ISO,
+        DateFormat.ISO_STRICT,
+        DateFormat.RFC,
+        DateFormat.SHORT,
+        DateFormat.RAW,
+        DateFormat.HUMAN,
+        DateFormat.UNIX,
+    ]
+
+
 class Defaults(object):
     """Read-only class for holding defaults that get overridden"""
 
     # These should match Git's defaults for git-defined values.
     autotemplate = False
     blame_viewer = 'git gui blame'
+    block_cursor = True
     bold_headers = False
     check_conflicts = True
     check_published_commits = True
@@ -75,6 +108,7 @@ class Defaults(object):
     merge_keep_backup = True
     merge_summary = True
     merge_verbosity = 2
+    mouse_zoom = True
     resize_browser_columns = False
     save_window_settings = True
     safe_mode = False
@@ -86,8 +120,10 @@ class Defaults(object):
     textwidth = 72
     theme = 'default'
     hidpi = hidpi.Option.AUTO
+    patches_directory = 'patches'
     status_indent = False
     status_show_totals = False
+    logdate = DateFormat.DEFAULT
 
 
 def blame_viewer(context):
@@ -96,6 +132,11 @@ def blame_viewer(context):
     return context.cfg.get(BLAME_VIEWER, default=default)
 
 
+def block_cursor(context):
+    """Should we display a block cursor in diff editors?"""
+    return context.cfg.get(BLOCK_CURSOR, default=Defaults.block_cursor)
+
+
 def bold_headers(context):
     """Should we bold the Status column headers?"""
     return context.cfg.get(BOLD_HEADERS, default=Defaults.bold_headers)
@@ -152,9 +193,12 @@ def fallback_editor():
 
 
 def _remap_editor(app):
-    """Remap a configured editorinto a visual editor name"""
-    # We do this for vim users because this configuration is convenient.
-    return {'vim': 'gvim -f'}.get(app, app)
+    """Remap a configured editor into a visual editor name"""
+    # We do this for vim users because this configuration is convenient for new users.
+    return {
+        'vim': 'gvim -f',
+        'nvim': 'nvim-qt --nofork',
+    }.get(app, app)
 
 
 def comment_char(context):
@@ -196,6 +240,11 @@ def linebreak(context):
     return context.cfg.get(LINEBREAK, default=Defaults.linebreak)
 
 
+def logdate(context):
+    """Return the configured log date format"""
+    return context.cfg.get(LOGDATE, default=Defaults.logdate)
+
+
 def maxrecent(context):
     """Return the configured maximum number of Recent Repositories"""
     value = Defaults.maxrecent
@@ -204,6 +253,11 @@ def maxrecent(context):
     return value
 
 
+def mouse_zoom(context):
+    """Should we zoom text when using Ctrl + MouseWheel scroll"""
+    return context.cfg.get(MOUSE_ZOOM, default=Defaults.mouse_zoom)
+
+
 def spellcheck(context):
     """Should we spellcheck commit messages?"""
     return context.cfg.get(SPELL_CHECK, default=Defaults.spellcheck)
@@ -214,6 +268,11 @@ def expandtab(context):
     return context.cfg.get(EXPANDTAB, default=Defaults.expandtab)
 
 
+def patches_directory(context):
+    """Return the patches output directory"""
+    return context.cfg.get(PATCHES_DIRECTORY, default=Defaults.patches_directory)
+
+
 def sort_bookmarks(context):
     """Should we sort bookmarks by name?"""
     return context.cfg.get(SORT_BOOKMARKS, default=Defaults.sort_bookmarks)
