@@ -1,4 +1,3 @@
-# flake8: noqa
 from __future__ import absolute_import, division, print_function, unicode_literals
 import sys
 import re
@@ -116,10 +115,9 @@ class MainWindow(standard.MainWindow):
 
     def init_window_size(self):
         """Set the window size on the first initial view"""
-        context = self.context
         if utils.is_darwin():
-            desktop = context.app.desktop()
-            self.resize(desktop.width(), desktop.height())
+            width, height = qtutils.desktop_size()
+            self.resize(width, height)
         else:
             self.showMaximized()
 
@@ -292,8 +290,8 @@ class Editor(QtWidgets.QWidget):
         try:
             core.write(self.filename, string)
             status = 0
-        except (OSError, IOError, ValueError) as e:
-            msg, details = utils.format_exception(e)
+        except (OSError, IOError, ValueError) as exc:
+            msg, details = utils.format_exception(exc)
             sys.stderr.write(msg + '\n\n' + details)
             status = 128
         return status
@@ -451,8 +449,8 @@ class RebaseTreeWidget(standard.DraggableTreeWidget):
         params = dag.DAG(oid, 2)
         repo = dag.RepoReader(context, params)
         commits = []
-        for c in repo.get():
-            commits.append(c)
+        for commit in repo.get():
+            commits.append(commit)
         if commits:
             commits = commits[-1:]
         self.commits_selected.emit(commits)
@@ -549,7 +547,6 @@ class ComboBox(QtWidgets.QComboBox):
 
 
 class RebaseTreeWidgetItem(QtWidgets.QTreeWidgetItem):
-
     ENABLED_COLUMN = 1
     COMMAND_COLUMN = 2
     OID_LENGTH = 7
