@@ -1,4 +1,5 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
+import shlex
 
 from qtpy.QtCore import Qt
 from qtpy.QtCore import Signal
@@ -52,6 +53,8 @@ def save_path(context, path, model):
 
 
 class Browser(standard.Widget):
+    """A repository branch file browser. Browses files provided by GitRepoModel"""
+
     # Read-only mode property
     mode = property(lambda self: self.model.mode)
 
@@ -86,7 +89,10 @@ class Browser(standard.Widget):
         msg += N_('Branch: %s') % branch
         self.setToolTip(msg)
 
-        scope = dict(project=self.model.project, branch=branch)
+        scope = {
+            'project': self.model.project,
+            'branch': branch,
+        }
         title = N_('%(project)s: %(branch)s - Browse') % scope
         if self.mode == self.model.mode_amend:
             title += ' %s' % N_('(Amending)')
@@ -211,7 +217,7 @@ class RepoTreeView(standard.TreeView):
         )
 
         self.action_terminal = common.terminal_action(
-            context, self, self.selected_paths
+            context, self, func=self.selected_paths
         )
 
         self.x_width = QtGui.QFontMetrics(self.font()).width('x')
@@ -550,11 +556,22 @@ class SaveBlob(cmds.ContextCommand):
         model = self.browse_model
         ref = '%s:%s' % (model.ref, model.relpath)
         with core.xopen(model.filename, 'wb') as fp:
-            status, _, _ = git.show(ref, _stdout=fp)
+            status, output, err = git.show(ref, _stdout=fp)
 
-        msg = N_('Saved "%(filename)s" from "%(ref)s" to "%(destination)s"') % dict(
-            filename=model.relpath, ref=model.ref, destination=model.filename
+        out = '# git show %s >%s\n%s' % (
+            shlex.quote(ref),
+            shlex.quote(model.filename),
+            output,
         )
+        Interaction.command(N_('Error Saving File'), 'git show', status, out, err)
+        if status != 0:
+            return
+
+        msg = N_('Saved "%(filename)s" from "%(ref)s" to "%(destination)s"') % {
+            'filename': model.relpath,
+            'ref': model.ref,
+            'destination': model.filename,
+        }
         Interaction.log_status(status, msg, '')
 
         Interaction.information(
@@ -646,7 +663,6 @@ class BrowseBranch(standard.Dialog):
 
 # pylint: disable=too-many-ancestors
 class GitTreeWidget(standard.TreeView):
-
     selection_changed = Signal()
     path_chosen = Signal(object)
 
