@@ -15,6 +15,7 @@ from .. import hotkeys
 from .. import icons
 from .. import textwrap
 from .. import qtutils
+from .. import spellcheck
 from ..interaction import Interaction
 from ..gitcmds import commit_message_path
 from ..i18n import N_
@@ -24,8 +25,8 @@ from ..qtutils import get
 from ..utils import Group
 from . import defs
 from .selectcommits import select_commits
-from .spellcheck import SpellCheckTextEdit
-from .text import HintedLineEdit
+from .spellcheck import SpellCheckLineEdit, SpellCheckTextEdit
+from .text import anchor_mode
 
 
 class CommitMessageEditor(QtWidgets.QFrame):
@@ -35,9 +36,12 @@ class CommitMessageEditor(QtWidgets.QFrame):
 
     def __init__(self, context, parent):
         QtWidgets.QFrame.__init__(self, parent)
+        cfg = context.cfg
         self.context = context
         self.model = model = context.model
         self.spellcheck_initialized = False
+        self.spellcheck = spellcheck.NorvigSpellCheck()
+        self.spellcheck.set_dictionary(cfg.get('cola.dictionary', None))
 
         self._linebreak = None
         self._textwidth = None
@@ -65,7 +69,6 @@ class CommitMessageEditor(QtWidgets.QFrame):
 
         # Menu acctions
         self.menu_actions = menu_actions = [
-            None,
             self.signoff_action,
             self.commit_action,
             None,
@@ -74,19 +77,17 @@ class CommitMessageEditor(QtWidgets.QFrame):
             None,
             self.move_up,
             self.move_down,
+            None,
         ]
 
         # Widgets
-        self.summary = CommitSummaryLineEdit(context)
+        self.summary = CommitSummaryLineEdit(context, check=self.spellcheck)
         self.summary.setMinimumHeight(defs.tool_button_height)
         self.summary.menu_actions.extend(menu_actions)
 
-        cfg = context.cfg
-        self.summary_validator = MessageValidator(context, parent=self.summary)
-        self.summary.setValidator(self.summary_validator)
-
-        self.description = CommitMessageTextEdit(context, parent=self)
-        self.description.set_dictionary(cfg.get('cola.dictionary', None))
+        self.description = CommitMessageTextEdit(
+            context, check=self.spellcheck, parent=self
+        )
         self.description.menu_actions.extend(menu_actions)
 
         commit_button_tooltip = N_('Commit staged changes\nShortcut: Ctrl+Enter')
@@ -128,9 +129,9 @@ class CommitMessageEditor(QtWidgets.QFrame):
         # Spell checker
         self.check_spelling_action = self.actions_menu.addAction(N_('Check Spelling'))
         self.check_spelling_action.setCheckable(True)
-        spellcheck = prefs.spellcheck(context)
-        self.check_spelling_action.setChecked(spellcheck)
-        self.toggle_check_spelling(spellcheck)
+        spell_check = prefs.spellcheck(context)
+        self.check_spelling_action.setChecked(spell_check)
+        self.toggle_check_spelling(spell_check)
 
         # Line wrapping
         self.autowrap_action = self.actions_menu.addAction(N_('Auto-Wrap Lines'))
@@ -176,13 +177,8 @@ class CommitMessageEditor(QtWidgets.QFrame):
         # Handle the one-off autowrapping
         qtutils.connect_action_bool(self.autowrap_action, self.set_linebreak)
 
-        qtutils.add_action(
-            self.summary, N_('Move Down'), self.focus_description, *hotkeys.ACCEPT
-        )
-
-        qtutils.add_action(
-            self.summary, N_('Move Down'), self.summary_cursor_down, hotkeys.DOWN
-        )
+        self.summary.accepted.connect(self.focus_description)
+        self.summary.down_pressed.connect(self.summary_cursor_down)
 
         self.model.commit_message_changed.connect(
             self.set_commit_message, type=Qt.QueuedConnection
@@ -233,7 +229,7 @@ class CommitMessageEditor(QtWidgets.QFrame):
         QtCore.QTimer.singleShot(1, self.restore_size)
 
     def restore_size(self):
-        self.setMaximumHeight(2 ** 13)
+        self.setMaximumHeight(2**13)
 
     def focus_summary(self):
         self.summary.setFocus()
@@ -249,12 +245,7 @@ class CommitMessageEditor(QtWidgets.QFrame):
         subsequence "down" press moves to the end of the line.
 
         """
-        cur_position = self.summary.cursorPosition()
-        end_position = len(get(self.summary))
-        if cur_position == end_position:
-            self.focus_description()
-        else:
-            self.summary.setCursorPosition(end_position)
+        self.focus_description()
 
     def commit_message(self, raw=True):
         """Return the commit message as a unicode string"""
@@ -277,7 +268,7 @@ class CommitMessageEditor(QtWidgets.QFrame):
             return text
         return textwrap.word_wrap(text, self._tabwidth, self._textwidth)
 
-    def commit_summary_changed(self, value):
+    def commit_summary_changed(self):
         """Respond to changes to the `summary` field
 
         Newlines can enter the `summary` field when pasting, which is
@@ -286,6 +277,7 @@ class CommitMessageEditor(QtWidgets.QFrame):
         "extended description" field.
 
         """
+        value = self.summary.value()
         if '\n' in value:
             summary, description = value.split('\n', 1)
             description = description.lstrip('\n')
@@ -500,14 +492,14 @@ class CommitMessageEditor(QtWidgets.QFrame):
         commits = dag.RepoReader(context, params)
 
         menu_commits = []
-        for idx, c in enumerate(commits.get()):
-            menu_commits.insert(0, c)
+        for idx, commit in enumerate(commits.get()):
+            menu_commits.insert(0, commit)
             if idx > 5:
                 continue
 
         menu.clear()
-        for c in menu_commits:
-            menu.addAction(prefix + c.summary, cmds.run(cmd, context, c.oid))
+        for commit in menu_commits:
+            menu.addAction(prefix + commit.summary, cmds.run(cmd, context, commit.oid))
 
         if len(commits) == 6:
             menu.addSeparator()
@@ -531,7 +523,7 @@ class CommitMessageEditor(QtWidgets.QFrame):
         self.choose_commit(cmds.LoadFixupMessage)
 
     def toggle_check_spelling(self, enabled):
-        spellcheck = self.description.spellcheck
+        spell_check = self.spellcheck
         cfg = self.context.cfg
 
         if prefs.spellcheck(self.context) != enabled:
@@ -542,89 +534,88 @@ class CommitMessageEditor(QtWidgets.QFrame):
             user_name = cfg.get('user.name')
             if user_name:
                 for part in user_name.split():
-                    spellcheck.add_word(part)
+                    spell_check.add_word(part)
 
             # Add our email address to the dictionary
             user_email = cfg.get('user.email')
             if user_email:
                 for part in user_email.split('@'):
                     for elt in part.split('.'):
-                        spellcheck.add_word(elt)
+                        spell_check.add_word(elt)
 
             # git jargon
-            spellcheck.add_word('Acked')
-            spellcheck.add_word('Signed')
-            spellcheck.add_word('Closes')
-            spellcheck.add_word('Fixes')
+            spell_check.add_word('Acked')
+            spell_check.add_word('Signed')
+            spell_check.add_word('Closes')
+            spell_check.add_word('Fixes')
 
+        self.summary.highlighter.enable(enabled)
         self.description.highlighter.enable(enabled)
 
 
-class MessageValidator(QtGui.QValidator):
-    """Prevent invalid branch names"""
+# pylint: disable=too-many-ancestors
+class CommitSummaryLineEdit(SpellCheckLineEdit):
+    """Text input field for the commit summary"""
 
-    def __init__(self, context, parent=None):
-        super(MessageValidator, self).__init__(parent)
-        self.context = context
+    down_pressed = Signal()
+    accepted = Signal()
+
+    def __init__(self, context, check=None, parent=None):
+        hint = N_('Commit summary')
+        SpellCheckLineEdit.__init__(self, context, hint, check=check, parent=parent)
         self._comment_char = None
-        self.refresh()
-        context.cfg.updated.connect(self.refresh, type=Qt.QueuedConnection)
+        self._refresh_config()
 
-    def refresh(self):
+        self.textChanged.connect(self._update_summary_text, Qt.QueuedConnection)
+        context.cfg.updated.connect(self._refresh_config, type=Qt.QueuedConnection)
+
+    def _refresh_config(self):
         """Update comment char in response to config changes"""
         self._comment_char = prefs.comment_char(self.context)
 
-    def validate(self, string, idx):
-        """Scrub whitespace and validate the commit message"""
-        string = string.lstrip()
-        if string.startswith(self._comment_char):
-            state = self.Invalid
-        else:
-            state = self.Acceptable
-        return (state, string, idx)
+    def _update_summary_text(self):
+        """Prevent commit messages from starting with comment characters"""
+        value = self.value()
+        if self._comment_char and value.startswith(self._comment_char):
+            cursor = self.textCursor()
+            position = cursor.position()
 
+            value = value.lstrip()
+            if self._comment_char:
+                value = value.lstrip(self._comment_char).lstrip()
 
-class CommitSummaryLineEdit(HintedLineEdit):
+            self.set_value(value, block=True)
 
-    cursor = Signal(int, int)
+            value = self.value()
+            if position > 1:
+                position = max(0, min(position - 1, len(value) - 1))
+                cursor.setPosition(position)
+                self.setTextCursor(cursor)
 
-    def __init__(self, context, parent=None):
-        hint = N_('Commit summary')
-        HintedLineEdit.__init__(self, context, hint, parent=parent)
-        self.menu_actions = []
-
-    def build_menu(self):
-        menu = self.createStandardContextMenu()
-        add_menu_actions(menu, self.menu_actions)
-        return menu
-
-    def contextMenuEvent(self, event):
-        menu = self.build_menu()
-        menu.exec_(self.mapToGlobal(event.pos()))
+    def keyPressEvent(self, event):
+        """Allow "Enter" to focus into the extended description field"""
+        event_key = event.key()
+        if event_key in (
+            Qt.Key_Enter,
+            Qt.Key_Return,
+        ):
+            self.accepted.emit()
+            return
+        SpellCheckLineEdit.keyPressEvent(self, event)
 
 
 # pylint: disable=too-many-ancestors
 class CommitMessageTextEdit(SpellCheckTextEdit):
     leave = Signal()
 
-    def __init__(self, context, parent=None):
+    def __init__(self, context, check=None, parent=None):
         hint = N_('Extended description...')
-        SpellCheckTextEdit.__init__(self, context, hint, parent)
-        self.menu_actions = []
+        SpellCheckTextEdit.__init__(self, context, hint, check=check, parent=parent)
 
         self.action_emit_leave = qtutils.add_action(
             self, 'Shift Tab', self.leave.emit, hotkeys.LEAVE
         )
 
-    def build_menu(self):
-        menu, _ = self.context_menu()
-        add_menu_actions(menu, self.menu_actions)
-        return menu
-
-    def contextMenuEvent(self, event):
-        menu = self.build_menu()
-        menu.exec_(self.mapToGlobal(event.pos()))
-
     def keyPressEvent(self, event):
         if event.key() == Qt.Key_Up:
             cursor = self.textCursor()
@@ -635,8 +626,7 @@ class CommitMessageTextEdit(SpellCheckTextEdit):
                 # Otherwise, emit a signal so that the parent can
                 # change focus.
                 if cursor.hasSelection():
-                    cursor.setPosition(0)
-                    self.setTextCursor(cursor)
+                    self.set_cursor_position(0)
                 else:
                     self.leave.emit()
                 event.accept()
@@ -662,10 +652,8 @@ class CommitMessageTextEdit(SpellCheckTextEdit):
             text_after = all_text[position:]
             lines_after = text_after.count('\n')
             if lines_after == 0:
-                if event.modifiers() & Qt.ShiftModifier:
-                    mode = QtGui.QTextCursor.KeepAnchor
-                else:
-                    mode = QtGui.QTextCursor.MoveAnchor
+                select = event.modifiers() & Qt.ShiftModifier
+                mode = anchor_mode(select)
                 cursor.setPosition(len(all_text), mode)
                 self.setTextCursor(cursor)
                 event.accept()
@@ -674,14 +662,5 @@ class CommitMessageTextEdit(SpellCheckTextEdit):
 
     def setFont(self, font):
         SpellCheckTextEdit.setFont(self, font)
-        fm = self.fontMetrics()
-        self.setMinimumSize(QtCore.QSize(fm.width('MMMM'), fm.height() * 2))
-
-
-def add_menu_actions(menu, menu_actions):
-    """Add actions to a menu, treating None as a separator"""
-    for action in menu_actions:
-        if action is None:
-            menu.addSeparator()
-        else:
-            menu.addAction(action)
+        metrics = self.fontMetrics()
+        self.setMinimumSize(QtCore.QSize(metrics.width('MMMM'), metrics.height() * 2))
