@@ -1,5 +1,6 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 import re
+import time
 
 from qtpy import QtCore
 from qtpy import QtGui
@@ -90,13 +91,16 @@ class CompletionLineEdit(HintedLineEdit):
         Qt.Key_Down: 'down',
     }
 
-    def __init__(self, context, model_factory, hint='', parent=None):
+    def __init__(
+        self, context, model_factory, hint='', show_all_completions=False, parent=None
+    ):
         HintedLineEdit.__init__(self, context, hint, parent=parent)
         # Tracks when the completion popup was active during key events
 
         self.context = context
         # The most recently selected completion item
         self._selection = None
+        self._show_all_completions = show_all_completions
 
         # Create a completion model
         completion_model = model_factory(context, self)
@@ -207,7 +211,9 @@ class CompletionLineEdit(HintedLineEdit):
     def _completions_updated(self):
         popup = self.popup()
         if not popup.isVisible():
-            return
+            if not self.hasFocus() or not self._show_all_completions:
+                return
+            popup.show()
         # Select the first item
         idx = self._completion_model.index(0, 0)
         selection = QtCore.QItemSelection(idx, idx)
@@ -286,7 +292,6 @@ class CompletionLineEdit(HintedLineEdit):
 
 
 class GatherCompletionsThread(QtCore.QThread):
-
     items_gathered = Signal(object)
 
     def __init__(self, model):
@@ -297,15 +302,11 @@ class GatherCompletionsThread(QtCore.QThread):
 
     def dispose(self):
         self.running = False
-        try:
-            self.wait()
-        except RuntimeError:
-            # The C++ object may have already been deleted by python while
-            # the application is tearing down. This is fine.
-            pass
+        utils.catch_runtime_error(self.wait)
 
     def run(self):
         text = None
+        items = []
         self.running = True
         # Loop when the matched text changes between the start and end time.
         # This happens when gather_matches() takes too long and the
@@ -397,7 +398,6 @@ def ref_sort_key(ref):
 
 
 class CompletionModel(QtGui.QStandardItemModel):
-
     updated = Signal()
     items_gathered = Signal(object)
     model_updated = Signal()
@@ -429,11 +429,17 @@ class CompletionModel(QtGui.QStandardItemModel):
         if not self.update_thread.isRunning():
             self.update_thread.start()
 
-    # pylint: disable=unused-argument,no-self-use
+    # pylint: disable=unused-argument
     def gather_matches(self, case_sensitive):
         return ((), (), set())
 
     def apply_matches(self, match_tuple):
+        """Build widgets for all of the matching items"""
+        if not match_tuple:
+            # Results from background tasks may arrive after the widget
+            # has been destroyed.
+            utils.catch_runtime_error(self.set_items, [])
+            return
         matched_refs, matched_paths, dirs = match_tuple
         QStandardItem = QtGui.QStandardItem
 
@@ -457,28 +463,29 @@ class CompletionModel(QtGui.QStandardItemModel):
                 item.setIcon(from_filename(match))
             items.append(item)
 
-        try:
-            self.clear()
-            self.invisibleRootItem().appendRows(items)
-            self.updated.emit()
-        except RuntimeError:  # C++ object has been deleted
-            pass
+        # Results from background tasks can arrive after the widget has been destroyed.
+        utils.catch_runtime_error(self.set_items, items)
+
+    def set_items(self, items):
+        """Clear the widget and add items to the model"""
+        self.clear()
+        self.invisibleRootItem().appendRows(items)
+        self.updated.emit()
 
     def dispose(self):
         self.update_thread.dispose()
 
 
-def _identity(x):
-    return x
+def _identity(value):
+    return value
 
 
-def _lower(x):
-    return x.lower()
+def _lower(value):
+    return value.lower()
 
 
-def filter_matches(match_text, candidates, case_sensitive, sort_key=lambda x: x):
+def filter_matches(match_text, candidates, case_sensitive, sort_key=None):
     """Filter candidates and return the matches"""
-
     if case_sensitive:
         case_transform = _identity
     else:
@@ -490,13 +497,21 @@ def filter_matches(match_text, candidates, case_sensitive, sort_key=lambda x: x)
     else:
         matches = list(candidates)
 
-    matches.sort(key=lambda x: sort_key(case_transform(x)))
+    if case_sensitive:
+        if sort_key is None:
+            matches.sort()
+        else:
+            matches.sort(key=sort_key)
+    else:
+        if sort_key is None:
+            matches.sort(key=_lower)
+        else:
+            matches.sort(key=lambda x: sort_key(_lower(x)))
     return matches
 
 
 def filter_path_matches(match_text, file_list, case_sensitive):
     """Return matching completions from a list of candidate files"""
-
     files = set(file_list)
     files_and_dirs = utils.add_parents(files)
     dirs = files_and_dirs.difference(files)
@@ -511,6 +526,7 @@ class Completer(QtWidgets.QCompleter):
         self._model = model
         self.setCompletionMode(QtWidgets.QCompleter.UnfilteredPopupCompletion)
         self.setCaseSensitivity(Qt.CaseInsensitive)
+        self.setFilterMode(QtCore.Qt.MatchContains)
 
         model.model_updated.connect(self.update, type=Qt.QueuedConnection)
         self.setModel(model)
@@ -529,7 +545,7 @@ class GitCompletionModel(CompletionModel):
     def __init__(self, context, parent):
         CompletionModel.__init__(self, context, parent)
         self.context = context
-        context.model.updated.connect(self.model_updated)
+        context.model.updated.connect(self.model_updated, type=Qt.QueuedConnection)
 
     def gather_matches(self, case_sensitive):
         refs = filter_matches(
@@ -537,7 +553,6 @@ class GitCompletionModel(CompletionModel):
         )
         return (refs, (), set())
 
-    # pylint: disable=no-self-use
     def matches(self):
         return []
 
@@ -547,7 +562,7 @@ class GitRefCompletionModel(GitCompletionModel):
 
     def __init__(self, context, parent):
         GitCompletionModel.__init__(self, context, parent)
-        context.model.refs_updated.connect(self.model_updated)
+        context.model.refs_updated.connect(self.model_updated, type=Qt.QueuedConnection)
 
     def matches(self):
         model = self.context.model
@@ -625,7 +640,6 @@ class GitPathCompletionModel(GitCompletionModel):
     def __init__(self, context, parent):
         GitCompletionModel.__init__(self, context, parent)
 
-    # pylint: disable=no-self-use
     def candidate_paths(self):
         return []
 
@@ -673,18 +687,32 @@ class GitLogCompletionModel(GitRefCompletionModel):
 
     def __init__(self, context, parent):
         GitRefCompletionModel.__init__(self, context, parent)
-        self.model_updated.connect(self.gather_paths, type=Qt.QueuedConnection)
         self._paths = []
         self._model = context.model
+        self._runtask = qtutils.RunTask(parent=self)
+        self._time = 0.0  # ensure that the first event runs a task.
+        self.model_updated.connect(
+            self._start_gathering_paths, type=Qt.QueuedConnection
+        )
+
+    def _start_gathering_paths(self):
+        """Gather paths when the model changes"""
+        # Debounce updates that land within 1 second of each other.
+        if time.time() - self._time > 1.0:
+            self._runtask.start(qtutils.SimpleTask(self.gather_paths))
+        self._time = time.time()
 
     def gather_paths(self):
-        if not self._model.cfg.get(prefs.AUTOCOMPLETE_PATHS, True):
+        """Gather paths and store them in the model"""
+        self._time = time.time()
+        if self._model.cfg.get(prefs.AUTOCOMPLETE_PATHS, True):
+            self._paths = gitcmds.tracked_files(self.context)
+        else:
             self._paths = []
-            return
-        context = self.context
-        self._paths = gitcmds.tracked_files(context)
+        self._time = time.time()
 
     def gather_matches(self, case_sensitive):
+        """Filter paths and refs to find matching entries"""
         if not self._paths:
             self.gather_paths()
         refs = filter_matches(
@@ -704,12 +732,19 @@ class GitLogCompletionModel(GitRefCompletionModel):
         return (refs, paths, dirs)
 
 
-def bind_lineedit(model, hint=''):
+def bind_lineedit(model, hint='', show_all_completions=False):
     """Create a line edit bound against a specific model"""
 
     class BoundLineEdit(CompletionLineEdit):
         def __init__(self, context, hint=hint, parent=None):
-            CompletionLineEdit.__init__(self, context, model, hint=hint, parent=parent)
+            CompletionLineEdit.__init__(
+                self,
+                context,
+                model,
+                hint=hint,
+                show_all_completions=show_all_completions,
+                parent=parent,
+            )
             self.context = context
 
     return BoundLineEdit
@@ -719,7 +754,9 @@ def bind_lineedit(model, hint=''):
 GitLogLineEdit = bind_lineedit(GitLogCompletionModel, hint='<ref>')
 GitRefLineEdit = bind_lineedit(GitRefCompletionModel, hint='<ref>')
 GitCheckoutBranchLineEdit = bind_lineedit(
-    GitCheckoutBranchCompletionModel, hint='<branch>'
+    GitCheckoutBranchCompletionModel,
+    hint='<branch>',
+    show_all_completions=True,
 )
 GitCreateBranchLineEdit = bind_lineedit(GitCreateBranchCompletionModel, hint='<branch>')
 GitBranchLineEdit = bind_lineedit(GitBranchCompletionModel, hint='<branch>')
@@ -731,6 +768,7 @@ GitTrackedLineEdit = bind_lineedit(GitTrackedCompletionModel, hint='<path>')
 
 
 class GitDialog(QtWidgets.QDialog):
+    # The "lineedit" argument is provided by the derived class constructor.
     def __init__(self, lineedit, context, title, text, parent, icon=None):
         QtWidgets.QDialog.__init__(self, parent)
         self.context = context
@@ -748,8 +786,8 @@ class GitDialog(QtWidgets.QDialog):
             defs.no_margin,
             defs.button_spacing,
             qtutils.STRETCH,
-            self.ok_button,
             self.close_button,
+            self.ok_button,
         )
 
         self.main_layout = qtutils.vbox(
@@ -783,9 +821,9 @@ class GitDialog(QtWidgets.QDialog):
         dlg.show()
 
         def show_popup():
-            x = dlg.lineedit.x()
-            y = dlg.lineedit.y() + dlg.lineedit.height()
-            point = QtCore.QPoint(x, y)
+            x_val = dlg.lineedit.x()
+            y_val = dlg.lineedit.y() + dlg.lineedit.height()
+            point = QtCore.QPoint(x_val, y_val)
             mapped = dlg.mapToGlobal(point)
             dlg.lineedit.popup().move(mapped.x(), mapped.y())
             dlg.lineedit.popup().show()
