@@ -13,7 +13,6 @@ from .standard import TreeWidget
 
 # pylint: disable=too-many-ancestors
 class FileWidget(TreeWidget):
-
     files_selected = Signal(object)
     difftool_selected = Signal(object)
     histories_selected = Signal(object)
@@ -53,18 +52,29 @@ class FileWidget(TreeWidget):
         if not commits:
             self.clear()
             return
+
         git = self.context.git
-        commit = commits[0]
-        oid = commit.oid
-        status, out, _ = git.show(
-            oid, z=True, numstat=True, oneline=True, no_renames=True
-        )
-        if status == 0:
-            paths = [f for f in out.rstrip('\0').split('\0') if f]
-            if paths:
-                paths = paths[1:]
+        paths = []
+
+        if len(commits) > 1:
+            # Get a list of changed files for a commit range.
+            start = commits[0].oid + '~'
+            end = commits[-1].oid
+            status, out, _ = git.diff(start, end, z=True, numstat=True, no_renames=True)
+            if status == 0:
+                paths = [f for f in out.rstrip('\0').split('\0') if f]
         else:
-            paths = []
+            # Get the list of changed files in a single commit.
+            commit = commits[0]
+            oid = commit.oid
+            status, out, _ = git.show(
+                oid, z=True, numstat=True, oneline=True, no_renames=True
+            )
+            if status == 0:
+                paths = [f for f in out.rstrip('\0').split('\0') if f]
+                if paths:
+                    paths = paths[1:]  # Skip over the summary on the first line.
+
         self.list_files(paths)
 
     def list_files(self, files_log):
@@ -72,8 +82,8 @@ class FileWidget(TreeWidget):
         if not files_log:
             return
         files = []
-        for f in files_log:
-            item = FileTreeWidgetItem(f)
+        for filename in files_log:
+            item = FileTreeWidgetItem(filename)
             files.append(item)
         self.insertTopLevelItems(0, files)
 
@@ -92,8 +102,8 @@ class FileWidget(TreeWidget):
     def show(self):
         self.adjust_columns(QSize(), QSize())
 
-    def resizeEvent(self, e):
-        self.adjust_columns(e.size(), e.oldSize())
+    def resizeEvent(self, event):
+        self.adjust_columns(event.size(), event.oldSize())
 
     def contextMenuEvent(self, event):
         menu = qtutils.create_menu(N_('Actions'), self)
