@@ -36,6 +36,7 @@ from . import browse
 from . import cfgactions
 from . import clone
 from . import commitmsg
+from . import common
 from . import compare
 from . import createbranch
 from . import createtag
@@ -89,8 +90,9 @@ class MainView(standard.MainWindow):
             )
 
         # "Actions" widget
+        self.actionswidget = action.ActionButtons(context, self)
         self.actionsdock = create_dock(
-            'Actions', N_('Actions'), self, widget=action.ActionButtons(context, self)
+            'Actions', N_('Actions'), self, widget=self.actionswidget
         )
         qtutils.hide_dock(self.actionsdock)
 
@@ -99,7 +101,7 @@ class MainView(standard.MainWindow):
             'Status',
             N_('Status'),
             self,
-            fn=lambda dock: status.StatusWidget(context, dock.titleBarWidget(), dock),
+            func=lambda dock: status.StatusWidget(context, dock.titleBarWidget(), dock),
         )
         self.statuswidget = self.statusdock.widget()
 
@@ -108,7 +110,7 @@ class MainView(standard.MainWindow):
             'Favorites',
             N_('Favorites'),
             self,
-            fn=lambda dock: bookmarks.bookmark(context, dock),
+            func=lambda dock: bookmarks.bookmark(context, dock),
         )
         bookmarkswidget = self.bookmarksdock.widget()
         qtutils.hide_dock(self.bookmarksdock)
@@ -117,7 +119,7 @@ class MainView(standard.MainWindow):
             'Recent',
             N_('Recent'),
             self,
-            fn=lambda dock: bookmarks.recent(context, dock),
+            func=lambda dock: bookmarks.recent(context, dock),
         )
         recentwidget = self.recentdock.widget()
         qtutils.hide_dock(self.recentdock)
@@ -125,7 +127,10 @@ class MainView(standard.MainWindow):
 
         # "Branch" widgets
         self.branchdock = create_dock(
-            'Branches', N_('Branches'), self, fn=partial(branch.BranchesWidget, context)
+            'Branches',
+            N_('Branches'),
+            self,
+            func=partial(branch.BranchesWidget, context),
         )
         self.branchwidget = self.branchdock.widget()
         titlebar = self.branchdock.titleBarWidget()
@@ -137,7 +142,7 @@ class MainView(standard.MainWindow):
             'Submodules',
             N_('Submodules'),
             self,
-            fn=partial(submodules.SubmodulesWidget, context),
+            func=partial(submodules.SubmodulesWidget, context),
         )
         self.submoduleswidget = self.submodulesdock.widget()
 
@@ -149,8 +154,8 @@ class MainView(standard.MainWindow):
         self.position_label.setFont(font)
 
         # make the position label fixed size to avoid layout issues
-        fm = self.position_label.fontMetrics()
-        width = fm.width('99:999') + defs.spacing
+        metrics = self.position_label.fontMetrics()
+        width = metrics.width('99:999') + defs.spacing
         self.position_label.setMinimumWidth(width)
 
         editor = commitmsg.CommitMessageEditor(context, self)
@@ -168,7 +173,10 @@ class MainView(standard.MainWindow):
 
         # "Diff Viewer" widget
         self.diffdock = create_dock(
-            'Diff', N_('Diff'), self, fn=lambda dock: diff.Viewer(context, parent=dock)
+            'Diff',
+            N_('Diff'),
+            self,
+            func=lambda dock: diff.Viewer(context, parent=dock),
         )
         self.diffviewer = self.diffdock.widget()
         self.diffviewer.set_diff_type(self.model.diff_type)
@@ -232,6 +240,34 @@ class MainView(standard.MainWindow):
         )
         self.apply_patches_action.setIcon(icons.diff())
 
+        self.apply_patches_abort_action = add_action(
+            self,
+            N_('Abort Applying Patches...'),
+            cmds.run(cmds.AbortApplyPatch, context),
+        )
+        self.apply_patches_abort_action.setIcon(icons.style_dialog_discard())
+        self.apply_patches_abort_action.setToolTip(
+            N_('Abort the current "git am" patch session')
+        )
+
+        self.apply_patches_continue_action = add_action(
+            self,
+            N_('Continue Applying Patches'),
+            cmds.run(cmds.ApplyPatchesContinue, context),
+        )
+        self.apply_patches_continue_action.setToolTip(
+            N_('Commit the current state and continue applying patches')
+        )
+        self.apply_patches_continue_action.setIcon(icons.commit())
+
+        self.apply_patches_skip_action = add_action(
+            self, N_('Skip Current Patch'), cmds.run(cmds.ApplyPatchesContinue, context)
+        )
+        self.apply_patches_skip_action.setToolTip(
+            N_('Skip applying the current patch and continue applying patches')
+        )
+        self.apply_patches_skip_action.setIcon(icons.discard())
+
         self.export_patches_action = add_action(
             self,
             N_('Export Patches...'),
@@ -250,11 +286,11 @@ class MainView(standard.MainWindow):
         )
         self.new_bare_repository_action.setIcon(icons.new())
 
-        prefs_fn = partial(
+        prefs_func = partial(
             prefs_widget.preferences, context, parent=self, model=prefs_model
         )
         self.preferences_action = add_action(
-            self, N_('Preferences'), prefs_fn, QtGui.QKeySequence.Preferences
+            self, N_('Preferences'), prefs_func, QtGui.QKeySequence.Preferences
         )
         self.preferences_action.setIcon(icons.configure())
 
@@ -294,6 +330,10 @@ class MainView(standard.MainWindow):
             hotkeys.CHERRY_PICK,
         )
         self.cherry_pick_action.setIcon(icons.cherry_pick())
+        self.cherry_pick_abort_action = add_action(
+            self, N_('Abort Cherry-Pick...'), cmds.run(cmds.AbortCherryPick, context)
+        )
+        self.cherry_pick_abort_action.setIcon(icons.style_dialog_discard())
 
         self.load_commitmsg_action = add_action(
             self, N_('Load Commit Message...'), partial(guicmds.load_commitmsg, context)
@@ -327,7 +367,7 @@ class MainView(standard.MainWindow):
         self.merge_abort_action = add_action(
             self, N_('Abort Merge...'), cmds.run(cmds.AbortMerge, context)
         )
-        self.merge_abort_action.setIcon(icons.style_dialog_reset())
+        self.merge_abort_action.setIcon(icons.style_dialog_discard())
 
         self.update_submodules_action = add_action(
             self,
@@ -477,6 +517,18 @@ class MainView(standard.MainWindow):
             self, N_('About'), partial(about.about_dialog, context)
         )
 
+        self.diff_against_commit_action = add_action(
+            self,
+            N_('Against Commit... (Diff Mode)'),
+            partial(guicmds.diff_against_commit, context),
+        )
+        self.diff_against_commit_action.setIcon(icons.compare())
+
+        self.exit_diff_mode_action = add_action(
+            self, N_('Exit Diff Mode'), cmds.run(cmds.ResetMode, context)
+        )
+        self.exit_diff_mode_action.setIcon(icons.compare())
+
         self.diff_expression_action = add_action(
             self, N_('Expression...'), partial(guicmds.diff_expression, context)
         )
@@ -546,10 +598,12 @@ class MainView(standard.MainWindow):
             cmds.run(cmds.Rebase, context),
             hotkeys.REBASE_START_AND_CONTINUE,
         )
+        self.rebase_start_action.setIcon(icons.play())
 
         self.rebase_edit_todo_action = add_action(
             self, N_('Edit...'), cmds.run(cmds.RebaseEditTodo, context)
         )
+        self.rebase_edit_todo_action.setIcon(icons.edit())
 
         self.rebase_continue_action = add_action(
             self,
@@ -557,14 +611,17 @@ class MainView(standard.MainWindow):
             cmds.run(cmds.RebaseContinue, context),
             hotkeys.REBASE_START_AND_CONTINUE,
         )
+        self.rebase_continue_action.setIcon(icons.play())
 
         self.rebase_skip_action = add_action(
             self, N_('Skip Current Patch'), cmds.run(cmds.RebaseSkip, context)
         )
+        self.rebase_skip_action.setIcon(icons.delete())
 
         self.rebase_abort_action = add_action(
             self, N_('Abort'), cmds.run(cmds.RebaseAbort, context)
         )
+        self.rebase_abort_action.setIcon(icons.close())
 
         # For "Start Rebase" only, reverse the first argument to setEnabled()
         # so that we can operate on it as a group.
@@ -598,6 +655,18 @@ class MainView(standard.MainWindow):
             self, N_('Reset Layout'), self.reset_layout
         )
 
+        self.quick_repository_search = add_action(
+            self,
+            N_('Quick Open...'),
+            lambda: guicmds.open_quick_repo_search(self.context, parent=self),
+            hotkeys.OPEN_REPO_SEARCH,
+        )
+        self.quick_repository_search.setIcon(icons.search())
+
+        self.terminal_action = common.terminal_action(
+            context, self, hotkey=hotkeys.TERMINAL
+        )
+
         # Create the application menu
         self.menubar = QtWidgets.QMenuBar(self)
         self.setMenuBar(self.menubar)
@@ -605,6 +674,7 @@ class MainView(standard.MainWindow):
         # File Menu
         add_menu = qtutils.add_menu
         self.file_menu = add_menu(N_('&File'), self.menubar)
+        self.file_menu.addAction(self.quick_repository_search)
         # File->Open Recent menu
         self.open_recent_menu = self.file_menu.addMenu(N_('Open Recent'))
         self.open_recent_menu.setIcon(icons.folder())
@@ -620,10 +690,16 @@ class MainView(standard.MainWindow):
         self.file_menu.addAction(self.edit_remotes_action)
         self.file_menu.addAction(self.browse_recently_modified_action)
         self.file_menu.addSeparator()
-        self.file_menu.addAction(self.apply_patches_action)
-        self.file_menu.addAction(self.export_patches_action)
         self.file_menu.addAction(self.save_tarball_action)
 
+        self.patches_menu = self.file_menu.addMenu(N_('Patches'))
+        self.patches_menu.setIcon(icons.diff())
+        self.patches_menu.addAction(self.export_patches_action)
+        self.patches_menu.addAction(self.apply_patches_action)
+        self.patches_menu.addAction(self.apply_patches_continue_action)
+        self.patches_menu.addAction(self.apply_patches_skip_action)
+        self.patches_menu.addAction(self.apply_patches_abort_action)
+
         # Git Annex / Git LFS
         annex = core.find_executable('git-annex')
         lfs = core.find_executable('git-lfs')
@@ -675,10 +751,12 @@ class MainView(standard.MainWindow):
         )
         select_all.setIcon(icons.select_all())
         edit_menu.addSeparator()
-        commitmsg.add_menu_actions(edit_menu, self.commiteditor.menu_actions)
+        qtutils.add_menu_actions(edit_menu, self.commiteditor.menu_actions)
 
         # Actions menu
         self.actions_menu = add_menu(N_('Actions'), self.menubar)
+        if self.terminal_action is not None:
+            self.actions_menu.addAction(self.terminal_action)
         self.actions_menu.addAction(self.fetch_action)
         self.actions_menu.addAction(self.push_action)
         self.actions_menu.addAction(self.pull_action)
@@ -686,6 +764,7 @@ class MainView(standard.MainWindow):
         self.actions_menu.addSeparator()
         self.actions_menu.addAction(self.create_tag_action)
         self.actions_menu.addAction(self.cherry_pick_action)
+        self.actions_menu.addAction(self.cherry_pick_abort_action)
         self.actions_menu.addAction(self.merge_local_action)
         self.actions_menu.addAction(self.merge_abort_action)
         self.actions_menu.addSeparator()
@@ -718,8 +797,10 @@ class MainView(standard.MainWindow):
         self.diff_menu = add_menu(N_('Diff'), self.menubar)
         self.diff_menu.addAction(self.diff_expression_action)
         self.diff_menu.addAction(self.branch_compare_action)
-        self.diff_menu.addSeparator()
         self.diff_menu.addAction(self.show_diffstat_action)
+        self.diff_menu.addSeparator()
+        self.diff_menu.addAction(self.diff_against_commit_action)
+        self.diff_menu.addAction(self.exit_diff_mode_action)
 
         # Branch Menu
         self.branch_menu = add_menu(N_('Branch'), self.menubar)
@@ -766,8 +847,7 @@ class MainView(standard.MainWindow):
         self.view_menu.aboutToShow.connect(lambda: self.build_view_menu(self.view_menu))
         self.setup_dockwidget_view_menu()
         if utils.is_darwin():
-            # TODO or self.menubar.setNativeMenuBar(False)
-            # Since native OSX menu doesn't show empty entries
+            # The native macOS menu doesn't show empty entries.
             self.build_view_menu(self.view_menu)
 
         # Help Menu
@@ -1006,10 +1086,12 @@ class MainView(standard.MainWindow):
     def refresh(self):
         """Update the title with the current branch and directory name."""
         curbranch = self.model.currentbranch
-        curdir = core.getcwd()
         is_merging = self.model.is_merging
         is_rebasing = self.model.is_rebasing
+        is_applying_patch = self.model.is_applying_patch
+        is_cherry_picking = self.model.is_rebasing
 
+        curdir = core.getcwd()
         msg = N_('Repository: %s') % curdir
         msg += '\n'
         msg += N_('Branch: %s') % curbranch
@@ -1021,7 +1103,18 @@ class MainView(standard.MainWindow):
                 'Resolve conflicts, commit changes, and run:\n'
                 '    Rebase > Continue'
             )
-
+        elif is_applying_patch:
+            msg += '\n\n'
+            msg += N_(
+                'This repository has unresolved conflicts after applying a patch.\n'
+                'Resolve conflicts and commit changes.'
+            )
+        elif is_cherry_picking:
+            msg += '\n\n'
+            msg += N_(
+                'This repository is in the middle of a cherry-pick.\n'
+                'Resolve conflicts and commit changes.'
+            )
         elif is_merging:
             msg += '\n\n'
             msg += N_(
@@ -1037,7 +1130,11 @@ class MainView(standard.MainWindow):
             self.commit_amend_action.setChecked(False)
 
         self.commitdock.setToolTip(msg)
+
+        self.actionswidget.set_mode(self.mode)
         self.commiteditor.set_mode(self.mode)
+        self.statuswidget.set_mode(self.mode)
+
         self.update_actions()
 
     def refresh_window_title(self):
@@ -1046,18 +1143,28 @@ class MainView(standard.MainWindow):
 
         project = self.model.project
         curbranch = self.model.currentbranch
+        is_cherry_picking = self.model.is_cherry_picking
         is_merging = self.model.is_merging
         is_rebasing = self.model.is_rebasing
+        is_applying_patch = self.model.is_applying_patch
+        is_diff_mode = self.model.is_diff_mode()
+        is_amend_mode = self.mode == self.model.mode_amend
+
         prefix = uchr(0xAB)
         suffix = uchr(0xBB)
 
-        if is_rebasing:
-            alerts.append(N_('Rebasing'))
+        if is_amend_mode:
+            alerts.append(N_('Amending'))
+        elif is_diff_mode:
+            alerts.append(N_('Diff Mode'))
+        elif is_cherry_picking:
+            alerts.append(N_('Cherry-picking'))
         elif is_merging:
             alerts.append(N_('Merging'))
-
-        if self.mode == self.model.mode_amend:
-            alerts.append(N_('Amending'))
+        elif is_rebasing:
+            alerts.append(N_('Rebasing'))
+        elif is_applying_patch:
+            alerts.append(N_('Applying Patch'))
 
         if alerts:
             alert_text = (prefix + ' %s ' + suffix + ' ') % ', '.join(alerts)
@@ -1082,6 +1189,14 @@ class MainView(standard.MainWindow):
 
         self.annex_init_action.setEnabled(not self.model.annex)
         self.lfs_init_action.setEnabled(not self.model.lfs)
+        self.merge_abort_action.setEnabled(self.model.is_merging)
+        self.cherry_pick_abort_action.setEnabled(self.model.is_cherry_picking)
+        self.apply_patches_continue_action.setEnabled(self.model.is_applying_patch)
+        self.apply_patches_skip_action.setEnabled(self.model.is_applying_patch)
+        self.apply_patches_abort_action.setEnabled(self.model.is_applying_patch)
+
+        diff_mode = self.model.mode == self.model.mode_diff
+        self.exit_diff_mode_action.setEnabled(diff_mode)
 
     def update_menu_actions(self):
         # Enable the Prepare Commit Message action if the hook exists
@@ -1185,7 +1300,7 @@ class MainView(standard.MainWindow):
         self.dag = dag.git_dag(self.context, existing_view=self.dag)
 
     def show_cursor_position(self, rows, cols):
-        display = '%02d:%02d' % (rows, cols)
+        display_content = '%02d:%02d' % (rows, cols)
         css = """
             <style>
             .good {
@@ -1213,7 +1328,7 @@ class MainView(standard.MainWindow):
             cls = 'first-warning'
         else:
             cls = 'good'
-        div = '<div class="%s">%s</div>' % (cls, display)
+        div = '<div class="%s">%s</div>' % (cls, display_content)
         self.position_label.setText(css + div)
 
 
@@ -1247,9 +1362,9 @@ class FocusProxy(object):
 
         def callback():
             focus = self.focus(name)
-            fn = getattr(focus, name, None)
-            if fn:
-                fn()
+            func = getattr(focus, name, None)
+            if func:
+                func()
 
         return callback
 
@@ -1280,7 +1395,7 @@ def _install_config_actions(context, menu, names_and_shortcuts):
         return
     menu.addSeparator()
     cache = {}
-    for (name, shortcut) in names_and_shortcuts:
+    for name, shortcut in names_and_shortcuts:
         sub_menu, action_name = build_menus(name, menu, cache)
         callback = cmds.run(cmds.RunConfigAction, context, name)
         menu_action = sub_menu.addAction(action_name, callback)
