@@ -112,7 +112,7 @@ def set_widget_value(widget, value):
             widget.setText(value)
         elif isinstance(widget, QtWidgets.QCheckBox):
             widget.setChecked(value)
-        elif isinstance(widget, qtutils.ComboBox):
+        elif hasattr(widget, 'set_value'):
             widget.set_value(value)
 
 
@@ -122,6 +122,15 @@ class RepoFormWidget(FormWidget):
         self.name = QtWidgets.QLineEdit()
         self.email = QtWidgets.QLineEdit()
 
+        tooltip = N_(
+            'Default directory when exporting patches.\n'
+            'Relative paths are relative to the current repository.\n'
+            'Absolute path are used as-is.'
+        )
+        patches_directory = prefs.patches_directory(context)
+        self.patches_directory = standard.DirectoryPathLineEdit(patches_directory, self)
+        self.patches_directory.setToolTip(tooltip)
+
         self.diff_context = standard.SpinBox(value=5, mini=2, maxi=9995)
         self.merge_verbosity = standard.SpinBox(value=5, maxi=5)
         self.merge_summary = qtutils.checkbox(checked=True)
@@ -129,8 +138,13 @@ class RepoFormWidget(FormWidget):
         self.merge_diffstat = qtutils.checkbox(checked=True)
         self.display_untracked = qtutils.checkbox(checked=True)
         self.show_path = qtutils.checkbox(checked=True)
-        self.tabwidth = standard.SpinBox(value=8, maxi=42)
-        self.textwidth = standard.SpinBox(value=72, maxi=150)
+
+        self.logdate = qtutils.combo(prefs.date_formats())
+        tooltip = N_(
+            'The date-time format used when displaying dates in Git DAG.\n'
+            'This value is passed to git log --date=<format>'
+        )
+        self.logdate.setToolTip(tooltip)
 
         tooltip = N_('Detect conflict markers in unmerged files')
         self.check_conflicts = qtutils.checkbox(checked=True, tooltip=tooltip)
@@ -149,10 +163,10 @@ class RepoFormWidget(FormWidget):
 
         self.add_row(N_('User Name'), self.name)
         self.add_row(N_('Email Address'), self.email)
-        self.add_row(N_('Tab Width'), self.tabwidth)
-        self.add_row(N_('Text Width'), self.textwidth)
+        self.add_row(N_('Log Date Format'), self.logdate)
         self.add_row(N_('Merge Verbosity'), self.merge_verbosity)
         self.add_row(N_('Number of Diff Context Lines'), self.diff_context)
+        self.add_row(N_('Patches Directory'), self.patches_directory)
         self.add_row(N_('Summarize Merge Commits'), self.merge_summary)
         self.add_row(
             N_('Automatically Load Commit Message Template'), self.autotemplate
@@ -171,8 +185,11 @@ class RepoFormWidget(FormWidget):
         self.set_config(
             {
                 prefs.AUTOTEMPLATE: (self.autotemplate, Defaults.autotemplate),
+                prefs.AUTOCOMPLETE_PATHS: (
+                    self.autocomplete_paths,
+                    Defaults.autocomplete_paths,
+                ),
                 prefs.CHECK_CONFLICTS: (self.check_conflicts, Defaults.check_conflicts),
-                prefs.ENABLE_GRAVATAR: (self.enable_gravatar, Defaults.enable_gravatar),
                 prefs.CHECK_PUBLISHED_COMMITS: (
                     self.check_published_commits,
                     Defaults.check_published_commits,
@@ -182,19 +199,19 @@ class RepoFormWidget(FormWidget):
                     self.display_untracked,
                     Defaults.display_untracked,
                 ),
-                prefs.USER_NAME: (self.name, ''),
-                prefs.USER_EMAIL: (self.email, ''),
+                prefs.ENABLE_GRAVATAR: (self.enable_gravatar, Defaults.enable_gravatar),
+                prefs.LOGDATE: (self.logdate, Defaults.logdate),
                 prefs.MERGE_DIFFSTAT: (self.merge_diffstat, Defaults.merge_diffstat),
                 prefs.MERGE_SUMMARY: (self.merge_summary, Defaults.merge_summary),
                 prefs.MERGE_VERBOSITY: (self.merge_verbosity, Defaults.merge_verbosity),
-                prefs.SAFE_MODE: (self.safe_mode, Defaults.safe_mode),
-                prefs.AUTOCOMPLETE_PATHS: (
-                    self.autocomplete_paths,
-                    Defaults.autocomplete_paths,
+                prefs.PATCHES_DIRECTORY: (
+                    self.patches_directory,
+                    Defaults.patches_directory,
                 ),
+                prefs.SAFE_MODE: (self.safe_mode, Defaults.safe_mode),
                 prefs.SHOW_PATH: (self.show_path, Defaults.show_path),
-                prefs.TABWIDTH: (self.tabwidth, Defaults.tabwidth),
-                prefs.TEXTWIDTH: (self.textwidth, Defaults.textwidth),
+                prefs.USER_NAME: (self.name, ''),
+                prefs.USER_EMAIL: (self.email, ''),
             }
         )
 
@@ -204,7 +221,7 @@ class SettingsFormWidget(FormWidget):
         FormWidget.__init__(self, context, model, parent)
 
         self.fixed_font = QtWidgets.QFontComboBox()
-        self.font_size = standard.SpinBox(value=12, mini=8, maxi=192)
+        self.font_size = standard.SpinBox(value=12, mini=6, maxi=192)
 
         self.maxrecent = standard.SpinBox(maxi=99)
         self.tabwidth = standard.SpinBox(maxi=42)
@@ -217,6 +234,7 @@ class SettingsFormWidget(FormWidget):
         self.mergetool = QtWidgets.QLineEdit()
 
         self.linebreak = qtutils.checkbox()
+        self.mouse_zoom = qtutils.checkbox()
         self.keep_merge_backups = qtutils.checkbox()
         self.sort_bookmarks = qtutils.checkbox()
         self.save_window_settings = qtutils.checkbox()
@@ -226,6 +244,8 @@ class SettingsFormWidget(FormWidget):
 
         self.add_row(N_('Fixed-Width Font'), self.fixed_font)
         self.add_row(N_('Font Size'), self.font_size)
+        self.add_row(N_('Text Width'), self.textwidth)
+        self.add_row(N_('Tab Width'), self.tabwidth)
         self.add_row(N_('Editor'), self.editor)
         self.add_row(N_('History Browser'), self.historybrowser)
         self.add_row(N_('Blame Viewer'), self.blameviewer)
@@ -239,6 +259,7 @@ class SettingsFormWidget(FormWidget):
         self.add_row(N_('Save GUI Settings'), self.save_window_settings)
         self.add_row(N_('Resize File Browser columns'), self.resize_browser_columns)
         self.add_row(N_('Check spelling'), self.check_spelling)
+        self.add_row(N_('Ctrl+MouseWheel to Zoom'), self.mouse_zoom)
 
         self.set_config(
             {
@@ -269,6 +290,7 @@ class SettingsFormWidget(FormWidget):
                     Defaults.resize_browser_columns,
                 ),
                 prefs.SPELL_CHECK: (self.check_spelling, Defaults.spellcheck),
+                prefs.MOUSE_ZOOM: (self.mouse_zoom, Defaults.mouse_zoom),
             }
         )
 
@@ -313,6 +335,7 @@ class AppearanceFormWidget(FormWidget):
         self.bold_headers = qtutils.checkbox()
         self.status_show_totals = qtutils.checkbox()
         self.status_indent = qtutils.checkbox()
+        self.block_cursor = qtutils.checkbox(checked=True)
 
         self.add_row(N_('GUI theme'), self.theme)
         self.add_row(N_('Icon theme'), self.icon_theme)
@@ -320,6 +343,7 @@ class AppearanceFormWidget(FormWidget):
         self.add_row(N_('Bold on dark headers instead of italic'), self.bold_headers)
         self.add_row(N_('Show file counts in Status titles'), self.status_show_totals)
         self.add_row(N_('Indent Status paths'), self.status_indent)
+        self.add_row(N_('Use a block cursor in diff editors'), self.block_cursor)
 
         self.set_config(
             {
@@ -332,6 +356,7 @@ class AppearanceFormWidget(FormWidget):
                 prefs.STATUS_INDENT: (self.status_indent, Defaults.status_indent),
                 prefs.THEME: (self.theme, Defaults.theme),
                 prefs.ICON_THEME: (self.icon_theme, Defaults.icon_theme),
+                prefs.BLOCK_CURSOR: (self.block_cursor, Defaults.block_cursor),
             }
         )
 
