@@ -237,8 +237,6 @@ class RemoteActionDialog(standard.Dialog):
         self.options_layout = qtutils.hbox(
             defs.no_margin,
             defs.button_spacing,
-            self.close_button,
-            qtutils.STRETCH,
             self.force_checkbox,
             self.ff_only_checkbox,
             self.no_ff_checkbox,
@@ -247,6 +245,8 @@ class RemoteActionDialog(standard.Dialog):
             self.rebase_checkbox,
             self.upstream_checkbox,
             self.prompt_checkbox,
+            qtutils.STRETCH,
+            self.close_button,
             self.action_button,
         )
 
@@ -591,7 +591,10 @@ class RemoteActionDialog(standard.Dialog):
 
             if prompt and candidate not in self.model.remote_branches:
                 title = N_('Push')
-                args = dict(branch=branch, remote=remote)
+                args = {
+                    'branch': branch,
+                    'remote': remote,
+                }
                 msg = (
                     N_(
                         'Branch "%(branch)s" does not exist in "%(remote)s".\n'
@@ -636,9 +639,11 @@ class RemoteActionDialog(standard.Dialog):
 
     def action_completed(self, task):
         """Grab the results of the action and finish up"""
-        status, out, err = task.result
         self.buttons.setEnabled(True)
+        if not task.result or not isinstance(task.result, (list, tuple)):
+            return
 
+        status, out, err = task.result
         command = 'git %s' % self.action.lower()
         message = Interaction.format_command_status(command, status)
         details = Interaction.format_out_err(out, err)
