@@ -1,5 +1,6 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 from functools import partial
+import os
 import time
 
 from qtpy import QtCore
@@ -43,9 +44,7 @@ class WidgetMixin(object):
         self.move(x, y)
 
     def resize_to_desktop(self):
-        desktop = QtWidgets.QApplication.instance().desktop()
-        width = desktop.width()
-        height = desktop.height()
+        width, height = qtutils.desktop_size()
         if utils.is_darwin():
             self.resize(width, height)
         else:
@@ -303,23 +302,9 @@ class TreeMixin(object):
         was_collapsed = not was_expanded
 
         # Vim keybindings...
-        # Rewrite the event before marshalling to QTreeView.event()
-        key = event.key()
+        event = _create_vim_navigation_key_event(event)
 
-        # Remap 'H' to 'Left'
-        if key == Qt.Key_H:
-            event = QtGui.QKeyEvent(event.type(), Qt.Key_Left, event.modifiers())
-        # Remap 'J' to 'Down'
-        elif key == Qt.Key_J:
-            event = QtGui.QKeyEvent(event.type(), Qt.Key_Down, event.modifiers())
-        # Remap 'K' to 'Up'
-        elif key == Qt.Key_K:
-            event = QtGui.QKeyEvent(event.type(), Qt.Key_Up, event.modifiers())
-        # Remap 'L' to 'Right'
-        elif key == Qt.Key_L:
-            event = QtGui.QKeyEvent(event.type(), Qt.Key_Right, event.modifiers())
-
-        # Re-read the event key to take the remappings into account
+        # Read the updated event key to take the remappings into account
         key = event.key()
         if key == Qt.Key_Up:
             idxs = widget.selectedIndexes()
@@ -346,14 +331,12 @@ class TreeMixin(object):
 
         # Process non-root entries with valid parents only.
         elif key == Qt.Key_Left and index.parent().isValid():
-
             # File entries have rowCount() == 0
             model = widget.model()
-            if (
-                hasattr(model, 'itemFromIndex')
-                and model.itemFromIndex(index).rowCount() == 0
-            ):
-                widget.setCurrentIndex(index.parent())
+            if hasattr(model, 'itemFromIndex'):
+                item = model.itemFromIndex(index)
+                if hasattr(item, 'rowCount') and item.rowCount() == 0:
+                    widget.setCurrentIndex(index.parent())
 
             # Otherwise, do this for collapsed directories only
             elif was_collapsed:
@@ -389,12 +372,11 @@ class TreeMixin(object):
         widget = self.widget
         if hasattr(widget, 'selectedItems'):
             return widget.selectedItems()
+        if hasattr(widget, 'itemFromIndex'):
+            item_from_index = widget.itemFromIndex
         else:
-            if hasattr(widget, 'itemFromIndex'):
-                item_from_index = widget.itemFromIndex
-            else:
-                item_from_index = widget.model().itemFromIndex
-            return [item_from_index(i) for i in widget.selectedIndexes()]
+            item_from_index = widget.model().itemFromIndex
+        return [item_from_index(i) for i in widget.selectedIndexes()]
 
     def selected_item(self):
         """Return the first selected item"""
@@ -431,6 +413,24 @@ class TreeMixin(object):
                 widget.setColumnWidth(idx, value)
 
 
+def _create_vim_navigation_key_event(event):
+    """Support minimal Vim-like keybindings by rewriting the QKeyEvents"""
+    key = event.key()
+    # Remap 'H' to 'Left'
+    if key == Qt.Key_H:
+        event = QtGui.QKeyEvent(event.type(), Qt.Key_Left, event.modifiers())
+    # Remap 'J' to 'Down'
+    elif key == Qt.Key_J:
+        event = QtGui.QKeyEvent(event.type(), Qt.Key_Down, event.modifiers())
+    # Remap 'K' to 'Up'
+    elif key == Qt.Key_K:
+        event = QtGui.QKeyEvent(event.type(), Qt.Key_Up, event.modifiers())
+    # Remap 'L' to 'Right'
+    elif key == Qt.Key_L:
+        event = QtGui.QKeyEvent(event.type(), Qt.Key_Right, event.modifiers())
+    return event
+
+
 class DraggableTreeMixin(TreeMixin):
     """A tree widget with internal drag+drop reordering of rows
 
@@ -525,7 +525,6 @@ class Dialog(WidgetMixin, QtWidgets.QDialog):
         self.dispose()
         return self.Base.reject(self)
 
-    # pylint: disable=no-self-use
     def dispose(self):
         """Extension method for model deregistration in sub-classes"""
         return
@@ -647,40 +646,45 @@ class ProgressDialog(QtWidgets.QProgressDialog):
 
     def __init__(self, title, label, parent):
         QtWidgets.QProgressDialog.__init__(self, parent)
+        self._parent = parent
         if parent is not None:
             self.setWindowModality(Qt.WindowModal)
+
+        self.animation_thread = ProgressAnimationThread(label, self)
+        self.animation_thread.updated.connect(self.set_text, type=Qt.QueuedConnection)
+
         self.reset()
         self.setRange(0, 0)
         self.setMinimumDuration(0)
         self.setCancelButton(None)
         self.setFont(qtutils.default_monospace_font())
-        self.thread = ProgressAnimationThread(label, self)
-        self.thread.updated.connect(self.refresh, type=Qt.QueuedConnection)
-
         self.set_details(title, label)
 
     def set_details(self, title, label):
+        """Update the window title and progress label"""
         self.setWindowTitle(title)
         self.setLabelText(label + '     ')
-        self.thread.set_text(label)
+        self.animation_thread.set_text(label)
 
-    def refresh(self, txt):
+    def set_text(self, txt):
+        """Set the label text"""
         self.setLabelText(txt)
 
     def keyPressEvent(self, event):
+        """Customize keyPressEvent to remove the ESC key cancel feature"""
         if event.key() != Qt.Key_Escape:
             super(ProgressDialog, self).keyPressEvent(event)
 
-    def show(self):
+    def start(self):
+        """Start the animation thread and use a wait cursor"""
         QtWidgets.QApplication.setOverrideCursor(Qt.WaitCursor)
-        super(ProgressDialog, self).show()
-        self.thread.start()
+        self.animation_thread.start()
 
-    def hide(self):
+    def stop(self):
+        """Stop the animation thread and restore the normal cursor"""
+        self.animation_thread.stop()
+        self.animation_thread.wait()
         QtWidgets.QApplication.restoreOverrideCursor()
-        self.thread.stop()
-        self.thread.wait()
-        super(ProgressDialog, self).hide()
 
 
 class ProgressAnimationThread(QtCore.QThread):
@@ -736,10 +740,55 @@ class SpinBox(QtWidgets.QSpinBox):
 
         font = self.font()
         metrics = QtGui.QFontMetrics(font)
-        width = max(self.minimumWidth(), metrics.width('XXXXXX'))
+        width = max(self.minimumWidth(), metrics.width('MMMMMM'))
         self.setMinimumWidth(width)
 
 
+class DirectoryPathLineEdit(QtWidgets.QWidget):
+    """A combined line edit and file browser button"""
+
+    def __init__(self, path, parent):
+        QtWidgets.QWidget.__init__(self, parent)
+
+        self.line_edit = QtWidgets.QLineEdit()
+        self.line_edit.setText(path)
+
+        self.browse_button = qtutils.create_button(
+            tooltip=N_('Select directory'), icon=icons.folder()
+        )
+        layout = qtutils.hbox(
+            defs.no_margin,
+            defs.spacing,
+            self.browse_button,
+            self.line_edit,
+        )
+        self.setLayout(layout)
+
+        qtutils.connect_button(self.browse_button, self._select_directory)
+
+    def set_value(self, value):
+        """Set the path value"""
+        self.line_edit.setText(value)
+
+    def value(self):
+        """Return the current path value"""
+        return self.line_edit.text().strip()
+
+    def _select_directory(self):
+        """Open a file browser and select a directory"""
+        output_dir = qtutils.opendir_dialog(N_('Select directory'), self.value())
+        if not output_dir:
+            return
+        # Make the directory relative only if it the current directory or
+        # or subdirectory from the current directory.
+        current_dir = core.getcwd()
+        if output_dir == current_dir:
+            output_dir = '.'
+        elif output_dir.startswith(current_dir + os.sep):
+            output_dir = os.path.relpath(output_dir)
+        self.set_value(output_dir)
+
+
 def export_header_columns(widget, state):
     """Save QHeaderView column sizes"""
     columns = []
@@ -786,7 +835,6 @@ class MessageBox(Dialog):
         cancel_text=None,
         cancel_icon=None,
     ):
-
         Dialog.__init__(self, parent=parent)
 
         if parent:
