@@ -63,6 +63,12 @@ class StatusWidget(QtWidgets.QFrame):
         self.tree = StatusTreeWidget(context, parent=self)
         self.setFocusProxy(self.tree)
 
+        tooltip = N_('Exit "Diff" mode')
+        icon = icons.circle_slash_red()
+        self.exit_diff_mode_button = qtutils.create_action_button(
+            tooltip=tooltip, icon=icon, visible=False
+        )
+
         self.main_layout = qtutils.vbox(
             defs.no_margin, defs.no_spacing, self.filter_widget, self.tree
         )
@@ -72,10 +78,16 @@ class StatusWidget(QtWidgets.QFrame):
             self, tooltip, self.toggle_filter, hotkeys.FILTER
         )
 
+        titlebar.add_corner_widget(self.exit_diff_mode_button)
         titlebar.add_corner_widget(self.filter_button)
+
         qtutils.connect_button(self.filter_button, self.toggle_filter)
+        qtutils.connect_button(
+            self.exit_diff_mode_button, cmds.run(cmds.ResetMode, self.context)
+        )
 
     def toggle_filter(self):
+        """Toggle the paths filter"""
         shown = not self.filter_widget.isVisible()
         self.filter_widget.setVisible(shown)
         if shown:
@@ -84,17 +96,25 @@ class StatusWidget(QtWidgets.QFrame):
             self.tree.setFocus()
 
     def set_initial_size(self):
+        """Set the initial size of the status widget"""
         self.setMaximumWidth(222)
-        QtCore.QTimer.singleShot(1, lambda: self.setMaximumWidth(2 ** 13))
+        QtCore.QTimer.singleShot(1, lambda: self.setMaximumWidth(2**13))
 
     def refresh(self):
+        """Refresh the tree and rerun the diff to see updates"""
         self.tree.show_selection()
 
     def set_filter(self, txt):
+        """Set the filter text"""
         self.filter_widget.setVisible(True)
         self.filter_widget.text.set_value(txt)
         self.filter_widget.apply_filter()
 
+    def set_mode(self, mode):
+        """React to changes in model's editing mode"""
+        exit_diff_mode_visible = mode == self.context.model.mode_diff
+        self.exit_diff_mode_button.setVisible(exit_diff_mode_visible)
+
     def move_up(self):
         self.tree.move_up()
 
@@ -122,16 +142,18 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         self.setUniformRowHeights(True)
         self.setAnimated(True)
         self.setRootIsDecorated(False)
-        self.setDragEnabled(True)
         self.setAutoScroll(False)
+        self.setDragEnabled(True)
+        self.setDragDropMode(QtWidgets.QAbstractItemView.DragOnly)
+        self._alt_drag = False
 
         if not prefs.status_indent(context):
             self.setIndentation(0)
 
-        ok = icons.ok()
+        ok_icon = icons.ok()
         compare = icons.compare()
         question = icons.question()
-        self._add_toplevel_item(N_('Staged'), ok, hide=True)
+        self._add_toplevel_item(N_('Staged'), ok_icon, hide=True)
         self._add_toplevel_item(N_('Unmerged'), compare, hide=True)
         self._add_toplevel_item(N_('Modified'), compare, hide=True)
         self._add_toplevel_item(N_('Untracked'), question, hide=True)
@@ -195,7 +217,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         self.worktree_dir_action = common.worktree_dir_action(context, self)
 
         self.terminal_action = common.terminal_action(
-            context, self, self.selected_group
+            context, self, func=self.selected_group
         )
 
         self.up_action = qtutils.add_action(
@@ -214,6 +236,16 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
             hotkeys.MOVE_DOWN_SECONDARY,
         )
 
+        # Checkout the selected paths using "git checkout --ours".
+        self.checkout_ours_action = qtutils.add_action(
+            self, cmds.CheckoutOurs.name(), cmds.run(cmds.CheckoutOurs, context)
+        )
+
+        # Checkout the selected paths using "git checkout --theirs".
+        self.checkout_theirs_action = qtutils.add_action(
+            self, cmds.CheckoutTheirs.name(), cmds.run(cmds.CheckoutTheirs, context)
+        )
+
         self.copy_path_action = qtutils.add_action(
             self,
             N_('Copy Path to Clipboard'),
@@ -230,12 +262,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         )
         self.copy_relpath_action.setIcon(icons.copy())
 
-        self.copy_leading_path_action = qtutils.add_action(
-            self,
-            N_('Copy Leading Path to Clipboard'),
-            partial(copy_leading_path, context),
-        )
-        self.copy_leading_path_action.setIcon(icons.copy())
+        self.copy_leading_paths_value = 1
 
         self.copy_basename_action = qtutils.add_action(
             self, N_('Copy Basename to Clipboard'), partial(copy_basename, context)
@@ -392,7 +419,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         did_reselect = False
 
         with qtutils.BlockSignals(self):
-            for (new, old, sel, reselect) in saved_selection:
+            for new, old, sel, reselect in saved_selection:
                 for item in sel:
                     if item in new:
                         reselect(item, current=False)
@@ -409,7 +436,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         #
         # The following block of code implements the behavior of selecting
         # the next item based on the previous selection.
-        for (new, old, sel, reselect) in saved_selection:
+        for new, old, sel, reselect in saved_selection:
             # When modified is staged, select the next modified item
             # When unmerged is staged, select the next unmerged item
             # When unstaging, select the next staged item
@@ -471,7 +498,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
                 # if the modified or untracked headers are selected.
                 cmds.do(cmds.UnstageAll, context)
                 return  # Everything was unstaged. There's nothing more to be done.
-            elif is_modified and is_untracked:
+            if is_modified and is_untracked:
                 # If both modified and untracked headers are selected then
                 # stage everything.
                 cmds.do(cmds.StageModifiedAndUntracked, context)
@@ -480,7 +507,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
             # possibly a subset of the other category (eg. all modified and
             # some untracked).  We don't return here so that StageOrUnstage
             # gets a chance to run below.
-            elif is_modified:
+            if is_modified:
                 cmds.do(cmds.StageModified, context)
             elif is_untracked:
                 cmds.do(cmds.StageUntracked, context)
@@ -577,6 +604,13 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         can_revert_edits = bool(selected.staged or selected.modified)
         self.revert_unstaged_edits_action.setEnabled(can_revert_edits)
 
+        enabled = self.selection_model.filename() is not None
+        self.default_app_action.setEnabled(enabled)
+        self.parent_dir_action.setEnabled(enabled)
+        self.copy_path_action.setEnabled(enabled)
+        self.copy_relpath_action.setEnabled(enabled)
+        self.copy_basename_action.setEnabled(enabled)
+
     def _set_staged(self, items):
         """Adds items to the 'Staged' subtree."""
         with qtutils.BlockSignals(self):
@@ -661,7 +695,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
 
     def _create_context_menu(self):
         """Set up the status menu for the repo status tree."""
-        s = self.selection()
+        sel = self.selection()
         menu = qtutils.create_menu('Status', self)
         selected_indexes = self.selected_indexes()
         if selected_indexes:
@@ -670,12 +704,12 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
             if category == HEADER_IDX:
                 return self._create_header_context_menu(menu, idx)
 
-        if s.staged:
-            self._create_staged_context_menu(menu, s)
-        elif s.unmerged:
-            self._create_unmerged_context_menu(menu, s)
+        if sel.staged:
+            self._create_staged_context_menu(menu, sel)
+        elif sel.unmerged:
+            self._create_unmerged_context_menu(menu, sel)
         else:
-            self._create_unstaged_context_menu(menu, s)
+            self._create_unstaged_context_menu(menu, sel)
 
         if not menu.isEmpty():
             menu.addSeparator()
@@ -698,18 +732,38 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         enabled = self.selection_model.filename() is not None
         self.copy_path_action.setEnabled(enabled)
         self.copy_relpath_action.setEnabled(enabled)
-        self.copy_leading_path_action.setEnabled(enabled)
         self.copy_basename_action.setEnabled(enabled)
+
+        copy_menu = QtWidgets.QMenu(N_('Copy...'), menu)
         copy_icon = icons.copy()
+        copy_menu.setIcon(copy_icon)
+
+        copy_leading_path_action = QtWidgets.QWidgetAction(copy_menu)
+        copy_leading_path_action.setEnabled(enabled)
+
+        widget = CopyLeadingPathWidget(
+            N_('Copy Leading Path to Clipboard'), self.context, copy_menu
+        )
+
+        # Store the value of the leading paths spinbox so that the value does not reset
+        # everytime the menu is shown and recreated.
+        widget.set_value(self.copy_leading_paths_value)
+        widget.spinbox.valueChanged.connect(
+            partial(setattr, self, 'copy_leading_paths_value')
+        )
+        copy_leading_path_action.setDefaultWidget(widget)
+
+        # Copy the leading path when the action is activated.
+        qtutils.connect_action(
+            copy_leading_path_action,
+            lambda widget=widget: copy_leading_path(context, widget.value()),
+        )
 
         menu.addSeparator()
-        copy_menu = QtWidgets.QMenu(N_('Copy...'), menu)
         menu.addMenu(copy_menu)
-
-        copy_menu.setIcon(copy_icon)
         copy_menu.addAction(self.copy_path_action)
         copy_menu.addAction(self.copy_relpath_action)
-        copy_menu.addAction(self.copy_leading_path_action)
+        copy_menu.addAction(copy_leading_path_action)
         copy_menu.addAction(self.copy_basename_action)
 
         settings = Settings.read()
@@ -763,7 +817,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
             return self._create_staged_submodule_context_menu(menu, s)
 
         context = self.context
-        if self._model.unstageable():
+        if self._model.is_unstageable():
             action = menu.addAction(
                 icons.remove(),
                 N_('Unstage Selected'),
@@ -782,7 +836,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         if all_exist:
             menu.addAction(self.launch_difftool_action)
 
-        if self._model.undoable():
+        if self._model.is_undoable():
             menu.addAction(self.revert_unstaged_edits_action)
 
         menu.addAction(self.view_history_action)
@@ -823,6 +877,9 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         menu.addAction(self.launch_editor_action)
         menu.addAction(self.view_history_action)
         menu.addAction(self.view_blame_action)
+        menu.addSeparator()
+        menu.addAction(self.checkout_ours_action)
+        menu.addAction(self.checkout_theirs_action)
         return menu
 
     def _create_unstaged_context_menu(self, menu, s):
@@ -831,7 +888,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         if modified_submodule:
             return self._create_modified_submodule_context_menu(menu, s)
 
-        if self._model.stageable():
+        if self._model.is_stageable():
             action = menu.addAction(
                 icons.add(),
                 N_('Stage Selected'),
@@ -848,10 +905,10 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
             for i in self.staged()
         )
 
-        if all_exist and s.modified and self._model.stageable():
+        if all_exist and s.modified and self._model.is_stageable():
             menu.addAction(self.launch_difftool_action)
 
-        if s.modified and self._model.stageable() and self._model.undoable():
+        if s.modified and self._model.is_stageable() and self._model.is_undoable():
             menu.addSeparator()
             menu.addAction(self.revert_unstaged_edits_action)
 
@@ -898,7 +955,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
             )
             menu.addSeparator()
 
-        if self._model.stageable():
+        if self._model.is_stageable():
             menu.addSeparator()
             action = menu.addAction(
                 icons.add(),
@@ -1049,7 +1106,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
 
         selected_indexes = self.selected_indexes()
         if not selected_indexes:
-            if self._model.amending():
+            if self._model.is_amend_mode() or self._model.is_diff_mode():
                 cmds.do(cmds.SetDiffText, context, '')
             else:
                 cmds.do(cmds.ResetMode, context)
@@ -1062,8 +1119,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
             cls = {
                 STAGED_IDX: cmds.DiffStagedSummary,
                 MODIFIED_IDX: cmds.Diffstat,
-                # TODO implement UnmergedSummary
-                # UNMERGED_IDX: cmds.UnmergedSummary,
+                UNMERGED_IDX: cmds.UnmergedSummary,
                 UNTRACKED_IDX: cmds.UntrackedSummary,
             }.get(idx, cmds.Diffstat)
             cmds.do(cls, context)
@@ -1132,6 +1188,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
                 return
 
     def move_up(self):
+        """Select the item above the currently selected item"""
         idx = self.selected_idx()
         all_files = self.all_files()
         if idx is None:
@@ -1152,6 +1209,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
             self.select_by_index(len(all_files) - 1)
 
     def move_down(self):
+        """Select the item below the currently selected item"""
         idx = self.selected_idx()
         all_files = self.all_files()
         if idx is None:
@@ -1171,18 +1229,30 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         else:
             self.select_by_index(0)
 
+    def mousePressEvent(self, event):
+        """Keep track of whether to drag URLs or just text"""
+        self._alt_drag = event.modifiers() & Qt.AltModifier
+        return super(StatusTreeWidget, self).mousePressEvent(event)
+
+    def mouseMoveEvent(self, event):
+        """Keep track of whether to drag URLs or just text"""
+        self._alt_drag = event.modifiers() & Qt.AltModifier
+        return super(StatusTreeWidget, self).mouseMoveEvent(event)
+
     def mimeData(self, items):
         """Return a list of absolute-path URLs"""
         context = self.context
         paths = qtutils.paths_from_items(items, item_filter=_item_filter)
-        return qtutils.mimedata_from_paths(context, paths)
+        include_urls = not self._alt_drag
+        return qtutils.mimedata_from_paths(context, paths, include_urls=include_urls)
 
-    # pylint: disable=no-self-use
     def mimeTypes(self):
-        return qtutils.path_mimetypes()
+        """Return the mimetypes that this widget generates"""
+        return qtutils.path_mimetypes(include_urls=not self._alt_drag)
 
 
 def _item_filter(item):
+    """Filter items down to just those that exist on disk"""
     return not item.deleted and core.exists(item.path)
 
 
@@ -1213,14 +1283,17 @@ def copy_basename(context):
     qtutils.copy_path(basename, absolute=False)
 
 
-def copy_leading_path(context):
-    """Copy the selected leading path to the clipboard"""
+def copy_leading_path(context, strip_components):
+    """Peal off trailing path components and copy the current path to the clipboard"""
     filename = context.selection.filename()
-    dirname = os.path.dirname(filename)
-    qtutils.copy_path(dirname, absolute=False)
+    value = filename
+    for _ in range(strip_components):
+        value = os.path.dirname(value)
+    qtutils.copy_path(value, absolute=False)
 
 
 def copy_format(context, fmt):
+    """Add variables usable in the custom Copy format strings"""
     values = {}
     values['path'] = path = context.selection.filename()
     values['abspath'] = abspath = os.path.abspath(path)
@@ -1232,6 +1305,7 @@ def copy_format(context, fmt):
 
 
 def show_help(context):
+    """Display the help for the custom Copy format strings"""
     help_text = N_(
         r"""
         Format String Variables
@@ -1250,6 +1324,8 @@ def show_help(context):
 
 
 class StatusFilterWidget(QtWidgets.QWidget):
+    """Filter paths displayed by the Status tool"""
+
     def __init__(self, context, parent=None):
         QtWidgets.QWidget.__init__(self, parent)
         self.context = context
@@ -1271,6 +1347,7 @@ class StatusFilterWidget(QtWidgets.QWidget):
         widget.editingFinished.connect(self.apply_filter)
 
     def apply_filter(self):
+        """Apply the text filter to the model"""
         value = get(self.text)
         if value == self._filter:
             return
@@ -1287,6 +1364,8 @@ def customize_copy_actions(context, parent):
 
 
 class CustomizeCopyActions(standard.Dialog):
+    """A dialog for defining custom Copy actions and format strings"""
+
     def __init__(self, context, parent):
         standard.Dialog.__init__(self, parent=parent)
         self.setWindowTitle(N_('Custom Copy Actions'))
@@ -1336,6 +1415,7 @@ class CustomizeCopyActions(standard.Dialog):
         QtCore.QTimer.singleShot(0, self.reload_settings)
 
     def reload_settings(self):
+        """Update the view to match the current settings"""
         # Called once after the GUI is initialized
         settings = self.context.settings
         settings.load()
@@ -1352,16 +1432,19 @@ class CustomizeCopyActions(standard.Dialog):
                 table.setItem(rows, 1, fmt)
 
     def export_state(self):
+        """Export the current state into the saved settings"""
         state = super(CustomizeCopyActions, self).export_state()
         standard.export_header_columns(self.table, state)
         return state
 
     def apply_state(self, state):
+        """Restore state from the saved settings"""
         result = super(CustomizeCopyActions, self).apply_state(state)
         standard.apply_header_columns(self.table, state)
         return result
 
     def add(self):
+        """Add a custom Copy action and format string"""
         self.table.setFocus()
         rows = self.table.rowCount()
         self.table.setRowCount(rows + 1)
@@ -1386,6 +1469,7 @@ class CustomizeCopyActions(standard.Dialog):
             self.table.removeRow(row)
 
     def save(self):
+        """Save custom copy actions to the settings"""
         copy_formats = []
         for row in range(self.table.rowCount()):
             name = self.table.item(row, 0)
@@ -1407,6 +1491,7 @@ class CustomizeCopyActions(standard.Dialog):
         self.accept()
 
     def table_selection_changed(self):
+        """Update the enabled state of action buttons based on the current selection"""
         items = self.table.selectedItems()
         self.remove_button.setEnabled(bool(items))
 
@@ -1547,3 +1632,69 @@ def _transplant_selection_across_sections(
             select_modified(old_path, current=True)
         elif old_path in staged_paths:
             select_staged(old_path, current=True)
+
+
+class CopyLeadingPathWidget(QtWidgets.QWidget):
+    """A widget that holds a label and a spinbox for the number of paths to strip"""
+
+    def __init__(self, title, context, parent):
+        QtWidgets.QWidget.__init__(self, parent)
+        self.context = context
+        self.icon = QtWidgets.QLabel(self)
+        self.label = QtWidgets.QLabel(self)
+        self.spinbox = standard.SpinBox(value=1, mini=1, maxi=99, parent=self)
+        self.spinbox.setToolTip(N_('The number of leading paths to strip'))
+
+        icon = icons.copy()
+        pixmap = icon.pixmap(defs.default_icon, defs.default_icon)
+        self.icon.setPixmap(pixmap)
+        self.label.setText(title)
+
+        layout = qtutils.hbox(
+            defs.small_margin,
+            defs.titlebar_spacing,
+            self.icon,
+            self.label,
+            qtutils.STRETCH,
+            self.spinbox,
+        )
+        self.setLayout(layout)
+
+        theme = context.app.theme
+        highlight_rgb = theme.highlight_color_rgb()
+        text_rgb, highlight_text_rgb = theme.text_colors_rgb()
+        disabled_text_rgb = theme.disabled_text_color_rgb()
+
+        stylesheet = """
+            * {
+                show-decoration-selected: 1
+            }
+            QLabel {
+                color: %(text_rgb)s;
+                show-decoration-selected: 1
+            }
+            QLabel:hover {
+                color: %(highlight_text_rgb)s;
+                background-color: %(highlight_rgb)s;
+                background-clip: padding;
+                show-decoration-selected: 1
+            }
+            QLabel:disabled {
+                color: %(disabled_text_rgb)s;
+            }
+        """ % {
+            'disabled_text_rgb': disabled_text_rgb,
+            'text_rgb': text_rgb,
+            'highlight_text_rgb': highlight_text_rgb,
+            'highlight_rgb': highlight_rgb,
+        }
+
+        self.setStyleSheet(stylesheet)
+
+    def value(self):
+        """Return the current value of the spinbox"""
+        return self.spinbox.value()
+
+    def set_value(self, value):
+        """Set the spinbox value"""
+        self.spinbox.setValue(value)
