@@ -1,6 +1,7 @@
 """Text widgets"""
 # pylint: disable=unexpected-keyword-arg
 from __future__ import absolute_import, division, print_function, unicode_literals
+from functools import partial
 import math
 
 from qtpy import QtCore
@@ -12,7 +13,9 @@ from qtpy.QtCore import Signal
 from ..models import prefs
 from ..qtutils import get
 from .. import hotkeys
+from .. import icons
 from .. import qtutils
+from .. import utils
 from ..i18n import N_
 from . import defs
 
@@ -22,8 +25,8 @@ def get_stripped(widget):
 
 
 class LineEdit(QtWidgets.QLineEdit):
-
     cursor_changed = Signal(int, int)
+    esc_pressed = Signal()
 
     def __init__(self, parent=None, row=1, get_value=None, clear_button=False):
         QtWidgets.QLineEdit.__init__(self, parent)
@@ -32,6 +35,7 @@ class LineEdit(QtWidgets.QLineEdit):
             get_value = get_stripped
         self._get_value = get_value
         self.cursor_position = LineEditCursorPosition(self, row)
+        self.menu_actions = []
 
         if clear_button and hasattr(self, 'setClearButtonEnabled'):
             self.setClearButtonEnabled(True)
@@ -58,6 +62,12 @@ class LineEdit(QtWidgets.QLineEdit):
         self.setText(value)
         self.setCursorPosition(pos)
 
+    def keyPressEvent(self, event):
+        key = event.key()
+        if key == Qt.Key_Escape:
+            self.esc_pressed.emit()
+        super(LineEdit, self).keyPressEvent(event)
+
 
 class LineEditCursorPosition(object):
     """Translate cursorPositionChanged(int,int) into cursorPosition(int,int)"""
@@ -166,8 +176,8 @@ class BaseTextEditExtension(QtCore.QObject):
     def set_tabwidth(self, width):
         self._tabwidth = width
         font = self.widget.font()
-        fm = QtGui.QFontMetrics(font)
-        pixels = fm.width('M' * width)
+        metrics = QtGui.QFontMetrics(font)
+        pixels = metrics.width('M' * width)
         self.widget.setTabStopWidth(pixels)
 
     def selected_line(self):
@@ -189,7 +199,13 @@ class BaseTextEditExtension(QtCore.QObject):
     def has_selection(self):
         return self.cursor().hasSelection()
 
+    def selected_text(self):
+        """Return the selected text"""
+        _, selection = self.offset_and_selection()
+        return selection
+
     def offset_and_selection(self):
+        """Return the cursor offset and selected text"""
         cursor = self.cursor()
         offset = cursor.selectionStart()
         selection_text = cursor.selection().toPlainText()
@@ -204,19 +220,56 @@ class BaseTextEditExtension(QtCore.QObject):
                 cursor = widget.cursorForPosition(event.pos())
                 widget.setTextCursor(cursor)
 
+    def add_links_to_menu(self, menu):
+        """Add actions for opening URLs to a custom menu"""
+        links = self._get_links()
+        if links:
+            menu.addSeparator()
+        for url in links:
+            action = menu.addAction(N_('Open "%s"') % url)
+            action.setIcon(icons.external())
+            qtutils.connect_action(
+                action, partial(QtGui.QDesktopServices.openUrl, QtCore.QUrl(url))
+            )
+
+    def _get_links(self):
+        """Return http links on the current line"""
+        _, selection = self.offset_and_selection()
+        if selection:
+            line = selection
+        else:
+            line = self.selected_line()
+        if not line:
+            return []
+        return [
+            word for word in line.split() if word.startswith(('http://', 'https://'))
+        ]
+
+    def create_context_menu(self, event_pos):
+        """Create a context menu for a widget"""
+        menu = self.widget.createStandardContextMenu(event_pos)
+        qtutils.add_menu_actions(menu, self.widget.menu_actions)
+        self.add_links_to_menu(menu)
+        return menu
+
+    def context_menu_event(self, event):
+        """Default context menu event"""
+        event_pos = event.pos()
+        menu = self.widget.create_context_menu(event_pos)
+        menu.exec_(self.widget.mapToGlobal(event_pos))
+
     # For extension by sub-classes
 
-    # pylint: disable=no-self-use
     def init(self):
         """Called during init for class-specific settings"""
         return
 
-    # pylint: disable=no-self-use,unused-argument
+    # pylint: disable=unused-argument
     def set_textwidth(self, width):
         """Set the text width"""
         return
 
-    # pylint: disable=no-self-use,unused-argument
+    # pylint: disable=unused-argument
     def set_linebreak(self, brk):
         """Enable word wrapping"""
         return
@@ -232,14 +285,16 @@ class PlainTextEditExtension(BaseTextEditExtension):
 
 
 class PlainTextEdit(QtWidgets.QPlainTextEdit):
-
     cursor_changed = Signal(int, int)
     leave = Signal()
 
-    def __init__(self, parent=None, get_value=None, readonly=False):
+    def __init__(self, parent=None, get_value=None, readonly=False, options=None):
         QtWidgets.QPlainTextEdit.__init__(self, parent)
         self.ext = PlainTextEditExtension(self, get_value, readonly)
         self.cursor_position = self.ext.cursor_position
+        self.mouse_zoom = True
+        self.options = options
+        self.menu_actions = []
 
     def get(self):
         """Return the raw unicode value from Qt"""
@@ -253,15 +308,43 @@ class PlainTextEdit(QtWidgets.QPlainTextEdit):
         """Return a safe value, e.g. a stripped value"""
         return self.ext.value()
 
+    def offset_and_selection(self):
+        """Return the cursor offset and selected text"""
+        return self.ext.offset_and_selection()
+
     def set_value(self, value, block=False):
         self.ext.set_value(value, block=block)
 
+    def set_mouse_zoom(self, value):
+        """Enable/disable text zooming in response to ctrl + mousewheel scroll events"""
+        self.mouse_zoom = value
+
+    def set_options(self, options):
+        """Register an Options widget"""
+        self.options = options
+
+    def set_word_wrapping(self, enabled, update=False):
+        """Enable/disable word wrapping"""
+        if update and self.options is not None:
+            with qtutils.BlockSignals(self.options.enable_word_wrapping):
+                self.options.enable_word_wrapping.setChecked(enabled)
+        if enabled:
+            self.setWordWrapMode(QtGui.QTextOption.WordWrap)
+            self.setLineWrapMode(QtWidgets.QPlainTextEdit.WidgetWidth)
+        else:
+            self.setWordWrapMode(QtGui.QTextOption.NoWrap)
+            self.setLineWrapMode(QtWidgets.QPlainTextEdit.NoWrap)
+
     def has_selection(self):
         return self.ext.has_selection()
 
     def selected_line(self):
         return self.ext.selected_line()
 
+    def selected_text(self):
+        """Return the selected text"""
+        return self.ext.selected_text()
+
     def set_tabwidth(self, width):
         self.ext.set_tabwidth(width)
 
@@ -277,11 +360,148 @@ class PlainTextEdit(QtWidgets.QPlainTextEdit):
 
     def wheelEvent(self, event):
         """Disable control+wheelscroll text resizing"""
-        if event.modifiers() & Qt.ControlModifier:
+        if not self.mouse_zoom and (event.modifiers() & Qt.ControlModifier):
             event.ignore()
             return
         super(PlainTextEdit, self).wheelEvent(event)
 
+    def create_context_menu(self, event_pos):
+        """Create a custom context menu"""
+        return self.ext.create_context_menu(event_pos)
+
+    def contextMenuEvent(self, event):
+        """Custom contextMenuEvent() for building our custom context menus"""
+        self.ext.context_menu_event(event)
+
+
+class TextSearchWidget(QtWidgets.QWidget):
+    """The search dialog that displays over a text edit field"""
+
+    def __init__(self, widget, parent):
+        super(TextSearchWidget, self).__init__(parent)
+        self.setAutoFillBackground(True)
+        self._widget = widget
+        self._parent = parent
+
+        self.text = HintedDefaultLineEdit(N_('Find in diff'), parent=self)
+
+        self.prev_button = qtutils.create_action_button(
+            tooltip=N_('Find the previous occurrence of the phrase'), icon=icons.up()
+        )
+
+        self.next_button = qtutils.create_action_button(
+            tooltip=N_('Find the next occurrence of the phrase'), icon=icons.down()
+        )
+
+        self.match_case_checkbox = qtutils.checkbox(N_('Match Case'))
+        self.whole_words_checkbox = qtutils.checkbox(N_('Whole Words'))
+
+        self.close_button = qtutils.create_action_button(
+            tooltip=N_('Close the find bar'), icon=icons.close()
+        )
+
+        layout = qtutils.hbox(
+            defs.margin,
+            defs.button_spacing,
+            self.text,
+            self.prev_button,
+            self.next_button,
+            self.match_case_checkbox,
+            self.whole_words_checkbox,
+            qtutils.STRETCH,
+            self.close_button,
+        )
+        self.setLayout(layout)
+        self.setFocusProxy(self.text)
+
+        self.text.esc_pressed.connect(self.hide_search)
+        self.text.returnPressed.connect(self.search)
+        self.text.textChanged.connect(self.search)
+
+        self.search_next_action = qtutils.add_action(
+            parent,
+            N_('Find next item'),
+            self.search,
+            hotkeys.SEARCH_NEXT,
+        )
+        self.search_prev_action = qtutils.add_action(
+            parent,
+            N_('Find previous item'),
+            self.search_backwards,
+            hotkeys.SEARCH_PREV,
+        )
+
+        qtutils.connect_button(self.next_button, self.search)
+        qtutils.connect_button(self.prev_button, self.search_backwards)
+        qtutils.connect_button(self.close_button, self.hide_search)
+        qtutils.connect_checkbox(self.match_case_checkbox, lambda _: self.search())
+        qtutils.connect_checkbox(self.whole_words_checkbox, lambda _: self.search())
+
+    def search(self):
+        """Emit a signal with the current search text"""
+        self.search_text(backwards=False)
+
+    def search_backwards(self):
+        """Emit a signal with the current search text for a backwards search"""
+        self.search_text(backwards=True)
+
+    def hide_search(self):
+        """Hide the search window"""
+        self.hide()
+        self._parent.setFocus(True)
+
+    def find_flags(self, backwards):
+        """Return QTextDocument.FindFlags for the current search options"""
+        flags = QtGui.QTextDocument.FindFlags()
+        if backwards:
+            flags = flags | QtGui.QTextDocument.FindBackward
+        if self.match_case_checkbox.isChecked():
+            flags = flags | QtGui.QTextDocument.FindCaseSensitively
+        if self.whole_words_checkbox.isChecked():
+            flags = flags | QtGui.QTextDocument.FindWholeWords
+        return flags
+
+    def is_case_sensitive(self):
+        """Are we searching using a case-insensitive search?"""
+        return self.match_case_checkbox.isChecked()
+
+    def search_text(self, backwards=False):
+        """Search the diff text for the given text"""
+        text = self.text.get()
+        cursor = self._widget.textCursor()
+        if cursor.hasSelection():
+            selected_text = cursor.selectedText()
+            case_sensitive = self.is_case_sensitive()
+            if text_matches(case_sensitive, selected_text, text):
+                if backwards:
+                    position = cursor.selectionStart()
+                else:
+                    position = cursor.selectionEnd()
+            else:
+                if backwards:
+                    position = cursor.selectionEnd()
+                else:
+                    position = cursor.selectionStart()
+            cursor.setPosition(position)
+            self._widget.setTextCursor(cursor)
+
+        flags = self.find_flags(backwards)
+        if not self._widget.find(text, flags):
+            if backwards:
+                location = QtGui.QTextCursor.End
+            else:
+                location = QtGui.QTextCursor.Start
+            cursor.movePosition(location, QtGui.QTextCursor.MoveAnchor)
+            self._widget.setTextCursor(cursor)
+            self._widget.find(text, flags)
+
+
+def text_matches(case_sensitive, a, b):
+    """Compare text with case sensitivity taken into account"""
+    if case_sensitive:
+        return a == b
+    return a.lower() == b.lower()
+
 
 class TextEditExtension(BaseTextEditExtension):
     def init(self):
@@ -300,7 +520,6 @@ class TextEditExtension(BaseTextEditExtension):
 
 
 class TextEdit(QtWidgets.QTextEdit):
-
     cursor_changed = Signal(int, int)
     leave = Signal()
 
@@ -309,6 +528,7 @@ class TextEdit(QtWidgets.QTextEdit):
         self.ext = TextEditExtension(self, get_value, readonly)
         self.cursor_position = self.ext.cursor_position
         self.expandtab_enabled = False
+        self.menu_actions = []
 
     def get(self):
         """Return the raw unicode value from Qt"""
@@ -318,12 +538,22 @@ class TextEdit(QtWidgets.QTextEdit):
         """Return a safe value, e.g. a stripped value"""
         return self.ext.value()
 
+    def set_cursor_position(self, position):
+        """Set the cursor position"""
+        cursor = self.textCursor()
+        cursor.setPosition(position)
+        self.setTextCursor(cursor)
+
     def set_value(self, value, block=False):
         self.ext.set_value(value, block=block)
 
     def selected_line(self):
         return self.ext.selected_line()
 
+    def selected_text(self):
+        """Return the selected text"""
+        return self.ext.selected_text()
+
     def set_tabwidth(self, width):
         self.ext.set_tabwidth(width)
 
@@ -355,7 +585,16 @@ class TextEdit(QtWidgets.QTextEdit):
         cursor = self.textCursor()
         cursor.insertText(' ' * tabwidth)
 
+    def create_context_menu(self, event_pos):
+        """Create a custom context menu"""
+        return self.ext.create_context_menu(event_pos)
+
+    def contextMenuEvent(self, event):
+        """Custom contextMenuEvent() for building our custom context menus"""
+        self.ext.context_menu_event(event)
+
     def keyPressEvent(self, event):
+        """Override keyPressEvent to handle tab expansion"""
         expandtab = self.should_expandtab(event)
         if expandtab:
             self.expandtab()
@@ -364,6 +603,7 @@ class TextEdit(QtWidgets.QTextEdit):
             QtWidgets.QTextEdit.keyPressEvent(self, event)
 
     def keyReleaseEvent(self, event):
+        """Override keyReleaseEvent to special-case tab expansion"""
         expandtab = self.should_expandtab(event)
         if expandtab:
             event.ignore()
@@ -441,12 +681,12 @@ class HintWidget(QtCore.QObject):
         error_bg_rgb = qtutils.rgb_css(error_bg_color)
         error_fg_rgb = qtutils.rgb_css(error_fg_color)
 
-        env = dict(
-            name=widget.__class__.__name__,
-            error_fg_rgb=error_fg_rgb,
-            error_bg_rgb=error_bg_rgb,
-            hint_rgb=hint_rgb,
-        )
+        env = {
+            'name': widget.__class__.__name__,
+            'error_fg_rgb': error_fg_rgb,
+            'error_bg_rgb': error_bg_rgb,
+            'hint_rgb': hint_rgb,
+        }
 
         self._default_style = ''
 
@@ -528,7 +768,9 @@ class HintWidget(QtCore.QObject):
             style = self._hint_style
         else:
             style = self._default_style
-        QtCore.QTimer.singleShot(0, lambda: self._widget.setStyleSheet(style))
+        QtCore.QTimer.singleShot(
+            0, lambda: utils.catch_runtime_error(self._widget.setStyleSheet, style)
+        )
 
     def eventFilter(self, _obj, event):
         """Enable/disable hint-mode when focus changes"""
@@ -549,7 +791,13 @@ class HintWidget(QtCore.QObject):
     def focus_out(self):
         """Re-enable hint-mode when losing focus"""
         widget = self.widget()
-        if not get(widget):
+        valid, value = utils.catch_runtime_error(get, widget)
+        if not valid:
+            # The widget may have just been destroyed during application shutdown.
+            # We're receiving a focusOut event but the widget can no longer be used.
+            # This can be safely ignored.
+            return
+        if not value:
             self.enable(True)
 
 
@@ -562,11 +810,13 @@ class HintedPlainTextEdit(PlainTextEdit):
         )
         self.hint = HintWidget(self, hint)
         self.hint.init()
+        self.context = context
         self.setFont(qtutils.diff_font(context))
         self.set_tabwidth(prefs.tabwidth(context))
         # Refresh palettes when text changes
         # pylint: disable=no-member
         self.textChanged.connect(self.hint.refresh)
+        self.set_mouse_zoom(context.cfg.get(prefs.MOUSE_ZOOM, default=True))
 
     def set_value(self, value, block=False):
         """Set the widget text or enable hint mode when empty"""
@@ -583,6 +833,7 @@ class HintedTextEdit(TextEdit):
         TextEdit.__init__(
             self, parent=parent, get_value=get_value_hinted, readonly=readonly
         )
+        self.context = context
         self.hint = HintWidget(self, hint)
         self.hint.init()
         # Refresh palettes when text changes
@@ -949,4 +1200,4 @@ class LineNumbers(TextDecorator):
                 Qt.AlignRight | Qt.AlignVCenter,
                 number,
             )
-            block = block.next()  # pylint: disable=next-method-called
+            block = block.next()
