@@ -6,15 +6,17 @@ from qtpy.QtCore import Qt
 from qtpy import QtWidgets
 
 from ..i18n import N_
-from ..widgets import standard
 from .. import icons
 from .. import qtutils
-from .toolbarcmds import COMMANDS
 from . import defs
+from . import standard
+from .toolbarcmds import COMMANDS
+
 
 TREE_LAYOUT = {
     'Others': ['Others::LaunchEditor', 'Others::RevertUnstagedEdits'],
     'File': [
+        'File::QuickOpen',
         'File::NewRepo',
         'File::OpenRepo',
         'File::OpenRepoNewWindow',
@@ -131,8 +133,6 @@ class ToolBarState(object):
             if data['float']:
                 toolbar.setWindowFlags(Qt.Tool | Qt.FramelessWindowHint)
                 toolbar.move(data['x'], data['y'])
-            # TODO: handle changed width when exists more than one toolbar in
-            # an area
 
     def export_state(self):
         result = []
@@ -144,7 +144,8 @@ class ToolBarState(object):
             if toolbar_area == Qt.NoToolBarArea:
                 continue  # filter out removed toolbars
             items = [x.data() for x in toolbar.actions()]
-
+            # show_icons is for backwards compatibility with git-cola <= 3.11.0
+            show_icons = toolbar.toolbar_style() != ToolBar.STYLE_TEXT_ONLY
             result.append(
                 {
                     'name': toolbar.windowTitle(),
@@ -155,8 +156,7 @@ class ToolBarState(object):
                     'y': toolbar.pos().y(),
                     'width': toolbar.width(),
                     'height': toolbar.height(),
-                    # show_icons kept for backwards compatibility in git-cola <= 3.11.0
-                    'show_icons': toolbar.toolbar_style() != ToolBar.STYLE_TEXT_ONLY,
+                    'show_icons': show_icons,
                     'toolbar_style': toolbar.toolbar_style(),
                     'visible': toolbar.isVisible(),
                     'items': items,
@@ -243,6 +243,7 @@ class ToolBar(QtWidgets.QToolBar):
             title = N_(command['title'])
             callback = partial(command['action'], self.context)
 
+            tooltip = command.get('tooltip', None)
             icon = None
             command_icon = command.get('icon', None)
             if command_icon:
